# This file contains the Namespace class

from pathlib import Path
from re import compile

from MCpypack.recipe.recipe import Recipe

class Namespace:
    """
    Represents a namespace.
    """

    # Valid namespace regex
    _NAMESPACE_PATTERN = compile(r"^[a-z_][a-z0-9_-]*$")

    def __init__(self,
                 name: str,
                ) -> None:
        """
        Initialize namespace instance.

        Parameters
        ----------
        name:
            Name of the namespace.
        """

        if not self._NAMESPACE_PATTERN.match(name):
            raise ValueError(f"Invalid namespace name: '{name}'")

        self.name: str = name

        self.recipes: list[Recipe] = []

    def add_recipes(self, *recipes: Recipe) -> None:
        """
        Add recipes to namespace.

        Parameters
        ----------
        *recipes:
            Recipes to add.
        """

        # Ensure recipes do not share the same name
        existing_names: list[str] = [recipe.name for recipe in self.recipes]

        for recipe in recipes:
            if recipe.name in existing_names:
                raise ValueError(f"Recipe with name {recipe.name} already exists in namespace {self.name}")
            existing_names.append(recipe.name)
            self.recipes.append(recipe)

    def export(self, datapack_dir: Path) -> None:
        """
        Create namespace related content.

        Parameters
        ----------
        datapack_dir:
            Directory of the datapack in which the namespace will be created.
        """

        namespace_dir: Path = datapack_dir / "data" / self.name
        namespace_dir.mkdir(parents=True, exist_ok=True)

        # Handle recipes
        if self.recipes:
            for recipe in self.recipes:
                recipe.export(namespace_dir=namespace_dir)



# This file contains the Datapack class

from pathlib import Path # For export directory
from packaging.version import Version # For checking Minecraft Version
import shutil
import json
import zipfile

from .namespace import Namespace

class Datapack:
    """
    Represent a datatpack that can be exported.
    """

    def __init__(self,
                 name: str,
                 description: str,
                 version: str,
                 relative_icon_path: str = '',
                 relative_export_dir: str = "export",
                 ) -> None:
        """
        Initialize a Datapack instance.

        Parameters
        ----------
        name:
            Name of the datapack.
        description:
            Description of the datapack.
        version:
            Minecraft Version the datapack should be created for.
        relative_icon_path:
            Icon path of the datapack relative to the current working directory.
        relative_export_dir:
            Relative directory to where the datapack will be exported.
        """

        self.name: str = name
        self.version: Version = Version(version)
        self.version_value: int | float = self._get_version_value()
        self.description: str = description
        self.icon_path: Path | None = Path.cwd() / relative_icon_path if relative_icon_path else None
        self.export_dir: Path = Path.cwd() / relative_export_dir

        # Store namespaces added with 'add_namespace'
        self.namespaces: list[Namespace] = []

    def _get_version_value(self) -> int | float:
        """
        Map Minecraft version to datapack version value.

        Returns:
            int | float: Datapack version.
        """

        v: Version = self.version

        _VERSION_MAP = [
            ("1.13", "1.14.4", 4),
            ("1.15", "1.16.1", 5),
            ("1.16.2", "1.16.5", 6),
            ("1.17", "1.17.1", 7),
            ("1.18", "1.18.1", 8),
            ("1.18.2", "1.18.2", 9),
            ("1.19", "1.19.3", 10),
            ("1.19.4", "1.19.4", 12),
            ("1.20", "1.20.1", 15),
            ("1.20.2", "1.20.2", 18),
            ("1.20.3", "1.20.4", 26),
            ("1.20.5", "1.20.6", 41),
            ("1.21", "1.21.1", 48),
            ("1.21.2", "1.21.3", 57),
            ("1.21.4", "1.21.4", 61),
            ("1.21.5", "1.21.5", 71),
            ("1.21.6", "1.21.6", 80),
            ("1.21.7", "1.21.8", 81),
            ("1.21.9", "1.21.10", 88.0),
            ("1.21.11", "1.21.11", 94.1),
        ]

        for min_v, max_v, pack_format in _VERSION_MAP:
                if Version(min_v) <= v <= Version(max_v):
                    return pack_format

        # Minecraft Version not supported
        raise ValueError(f"Unsupported version of Minecraft: {v}")

    def add_namespaces(self, *namespaces: Namespace) -> None:
        """
        Add namespace to datapack.

        Parameters
        ----------
        *namespaces:
            Add one or more namespaces to the datapack.
        """
        
        # Ensure namespaces do not have the same name
        existing_namespaces: list[str] = [namespace.name for namespace in self.namespaces]

        for namespace in namespaces:
            if namespace.name in existing_namespaces:
                raise ValueError(f"Namespace with name {namespace.name} already exists in Datapack {self.name}")
            existing_namespaces.append(namespace.name)
            self.namespaces.append(namespace)

    def export(self, overwrite: bool = True, zip: bool = False) -> None:
        """
        Export the datapack to location specified in 'relative_export_dir'.

        Creates the export directory if it does not exist.

        Parameters
        ----------
        overwrite:
            Delete the current version of the datapack and create the new one or
            panic if there is already a version of the datapack present.
            If set to 'False' append a number to the name if it already exists.
        zip:
            Export the datapack as a zip file.
        """

        # Create export directory
        self.export_dir.mkdir(parents=True, exist_ok=True)

        # Create datapack directory
        datapack_dir: Path = self.export_dir / self.name

        # Delete previous version of the datapack if present
        if overwrite:
            if datapack_dir.exists():
                shutil.rmtree(datapack_dir)
        else:
            counter: int = 1
            while datapack_dir.exists():
                datapack_dir = self.export_dir / f"{self.name}_{counter}"
                counter += 1

        # Now safe to create directory
        datapack_dir.mkdir(parents=True, exist_ok=False)

        # Create pack.mcmeta file
        pack_mcmeta_content: dict[str, dict[str, str | int | float]] = {
            "pack": {
                "description": self.description,
                "min_format": self.version_value,
                "max_format": self.version_value
            }
        }

        pack_mcmeta_file: Path = datapack_dir / "pack.mcmeta"
        with open(pack_mcmeta_file, "w", encoding="utf-8") as file:
            json.dump(pack_mcmeta_content, file, indent=4)

        # Copy icon into datapack
        if self.icon_path and self.icon_path.is_file():
            destination: Path = datapack_dir / "pack.png"
            shutil.copy(self.icon_path, destination)

        # Create namespaces
        if self.namespaces:
            for namespace in self.namespaces:
                namespace.export(datapack_dir=datapack_dir)

        # Create zip if 'zip' is set to 'True'
        if zip:
            zip_file_path: Path = self.export_dir / f"{datapack_dir.name}.zip"
            with zipfile.ZipFile(zip_file_path, "w",
                                 compression=zipfile.ZIP_DEFLATED) as zipf:
                for file_path in datapack_dir.rglob("*"):
                    zipf.write(file_path,
                               arcname=file_path.relative_to(datapack_dir.parent))



from MCpypack.item import Item

from .utils import CountedResult
from .recipe import Recipe

class Stonecutting(Recipe):
    """
    Stonecutting recipe.
    """

    @property
    def TYPE(cls) -> str:
        return "stonecutting"

    def __init__(self,
                 name: str,
                 ingredient: Item | list[Item],
                 result: CountedResult,
                 ) -> None:
        """
        Init stonecutting recipe.

        Parameters
        ----------
        name:
            Name of the recipe.
        ingredient:
            Ingredient of the stonecutting.
        result:
            Result of the crafting stored as a Result instance.
        """
        super().__init__(name)

        if isinstance(ingredient, list):
            self.config["ingredient"] = [i.value for i in ingredient]
        else:
            self.config["ingredient"] = ingredient.value

        self.config["result"] = result.to_dict()




from MCpypack.item import Item

from .utils import Category, CategoryLike, Group, CountedResult
from .recipe import Recipe

class CraftingShapeless(Recipe):
    """
    Shapeless crafting recipe.
    """

    @property
    def TYPE(cls) -> str:
        return "minecraft:crafting_shapeless"

    def __init__(self,
                 name: str,
                 ingredients: list[Item] | list[list[Item]] | Item | list[list[Item] | Item],
                 result: CountedResult,
                 group: Group | None = None,
                 category: CategoryLike = Category.MISC,
                 ) -> None:
        """
        Init shapeless crafting recipe.

        Parameters
        ----------
        name:
            Name of the recipe.
        ingredients:
            List of ingredients for the recipe.
        result:
            Result of the crafting stored as a Result instance.
        group:
            Optional.
            String identifier for grouping recipes.
        category:
            Recipe book category.
            Default is "misc".
        """
        super().__init__(name)

        # Convert category to Category enum if it is a string
        # Ensure valid value if string
        category_final: str = str(Category.from_str(category))
        

        # Convert ingredients into useful list
        ingredients_final: list[str] | list[list[str]] | list[list[str] | str]

        if isinstance(ingredients, Item):
            # Plain item and not a list of ingredients
            # -> Just convert it into a list with just the one value
            ingredients_final = [ingredients.value]

        elif isinstance(ingredients, list):
            # Ingredients is not just one value but rather a list
            # The list may contains just items or lists of items

            ingredients_final = []

            for ingredient in ingredients:
                if isinstance(ingredient, Item):
                    # If it is just an item append it to the list
                    ingredients_final.append(ingredient.value)

                elif isinstance(ingredient, list):
                    # If it is another list of type list[Item]
                    # -> Put each element into a list and then append it to the
                    #    final list
                    inner: list[str] = []
                    for sub in ingredient:
                        if isinstance(sub, Item):
                            inner.append(sub.value)
                    ingredients_final.append(inner)

        self.config["category"] = category_final
        self.config["ingredients"] = ingredients_final
        self.config["result"] = result.to_dict()

        if group:
            self.config["group"] = group



# This file contains the Category enum used to represent the category in which
# each recipe is put into
# It also contains the CategoryLike type which is the Category type or the
# str type

from enum import StrEnum
from typing import TypeAlias

class Category(StrEnum):
    """
    Enum for different recipe book categories.
    Available values are "equipment", "building", "misc", and "redstone".
    """

    EQUIPMENT = "equipment"
    BUILDING = "building"
    MISC = "misc"
    REDSTONE = "redstone"

    def __str__(self) -> str:
        """
        Return string representation of current value.
        """

        return self.value

    @classmethod
    def values(cls) -> list[str]:
        """
        Return a list of all possible category values.
        """

        return [c.value for c in cls]

    @classmethod
    def from_str(cls, value: str) -> Category:
        """
        Turn string into a possible value or raise ValueError.

        Parameters
        ----------
        value:
            String which will be converted into Category enum.
        """

        try:
            return cls(value)
        except ValueError:
            raise ValueError(f"Invalid category: {value}")

CategoryLike: TypeAlias = Category | str



from MCpypack.item import Item
from .utils import Category, CategoryLike, Group, CountedResult
from .recipe import Recipe

class CraftingShaped(Recipe):
    """
    Shaped crafting recipe.
    """

    @property
    def TYPE(self) -> str:
        return "minecraft:crafting_shaped"

    def __init__(self,
                 name: str,
                 pattern: list[str],
                 key: dict[str, Item] | dict[str, list[Item]] | dict[str, list[Item] | Item],
                 result: CountedResult,
                 group: Group | None = None,
                 category: CategoryLike = Category.MISC) -> None:

        super().__init__(name)

        # Validate keys
        pattern_keys = {char for row in pattern for char in row if char != " "}
        used_keys = set(key.keys())
        if pattern_keys != used_keys:
            raise ValueError(f"Pattern keys {pattern_keys} and used keys {used_keys} do not match.")

        # Convert category
        category_final = str(Category.from_str(category))

        # Convert key values inline: single Item -> str, list[Item] -> list[str]
        key_final: dict[str, str | list[str]] = {}
        for k, v in key.items():
            if isinstance(v, list):
                key_final[k] = [i.value for i in v]
            else:
                key_final[k] = v.value

        # Assign config
        self.config["category"] = category_final
        self.config["key"] = key_final
        self.config["pattern"] = pattern
        self.config["result"] = result.to_dict()
        if group:
            self.config["group"] = group



# This file contains the CampfireCooking class

from MCpypack.item import Item
from .utils import SimpleResult, Time, Experience
from .recipe import Recipe

class CampfireCooking(Recipe):
    """
    Campfire cooking recipe.
    """

    @property
    def TYPE(cls) -> str:
        return "minecraft:campfire_cooking"

    def __init__(self,
                 name: str,
                 ingredient: Item | list[Item],
                 result: SimpleResult,
                 cookingtime: Time | None = None,
                 experience: Experience | None = None,
                 ) -> None:
        """
        Init campfire cooking recipe.

        Parameters
        ----------
        name:
            Name of the recipe.
        ingredient:
            Ingredient of the recipe.
        result:
            Result of the cooking.
        cookingtime:
            Optional. Cookingtime in real-life time values.
        experience:
            Optional. The output experience of the recipe.
        """

        super().__init__(name)

        if isinstance(ingredient, list):
            self.config["ingredient"] = [i.value for i in ingredient]
        else:
            self.config["ingredient"] = ingredient.value

        self.config["result"] = result.to_dict()

        if experience:
            self.config["experience"] = experience

        if cookingtime:
            self.config["cookingtime"] = cookingtime.ticks.value



from enum import Enum


class Item(Enum):
    ACACIA_BOAT = "minecraft:acacia_boat"
    ACACIA_BUTTON = "minecraft:acacia_button"
    ACACIA_CHEST_BOAT = "minecraft:acacia_chest_boat"
    ACACIA_DOOR = "minecraft:acacia_door"
    ACACIA_FENCE_GATE = "minecraft:acacia_fence_gate"
    ACACIA_FENCE = "minecraft:acacia_fence"
    ACACIA_HANGING_SIGN = "minecraft:acacia_hanging_sign"
    ACACIA_LEAVES = "minecraft:acacia_leaves"
    ACACIA_LOG = "minecraft:acacia_log"
    ACACIA_PLANKS = "minecraft:acacia_planks"
    ACACIA_PRESSURE_PLATE = "minecraft:acacia_pressure_plate"
    ACACIA_SAPLING = "minecraft:acacia_sapling"
    ACACIA_SHELF = "minecraft:acacia_shelf"
    ACACIA_SIGN = "minecraft:acacia_sign"
    ACACIA_SLAB = "minecraft:acacia_slab"
    ACACIA_STAIRS = "minecraft:acacia_stairs"
    ACACIA_TRAPDOOR = "minecraft:acacia_trapdoor"
    ACACIA_WOOD = "minecraft:acacia_wood"
    ACTIVATOR_RAIL = "minecraft:activator_rail"
    AIR = "minecraft:air"
    ALLAY_SPAWN_EGG = "minecraft:allay_spawn_egg"
    ALLIUM = "minecraft:allium"
    AMETHYST_BLOCK = "minecraft:amethyst_block"
    AMETHYST_CLUSTER = "minecraft:amethyst_cluster"
    AMETHYST_SHARD = "minecraft:amethyst_shard"
    ANCIENT_DEBRIS = "minecraft:ancient_debris"
    ANDESITE = "minecraft:andesite"
    ANDESITE_SLAB = "minecraft:andesite_slab"
    ANDESITE_STAIRS = "minecraft:andesite_stairs"
    ANDESITE_WALL = "minecraft:andesite_wall"
    ANGLER_POTTERY_SHERD = "minecraft:angler_pottery_sherd"
    ANVIL = "minecraft:anvil"
    APPLE = "minecraft:apple"
    ARCHER_POTTERY_SHERD = "minecraft:archer_pottery_sherd"
    ARMADILLO_SCUTE = "minecraft:armadillo_scute"
    ARMADILLO_SPAWN_EGG = "minecraft:armadillo_spawn_egg"
    ARMOR_STAND = "minecraft:armor_stand"
    ARMS_UP_POTTERY_SHERD = "minecraft:arms_up_pottery_sherd"
    ARROW = "minecraft:arrow"
    AXOLOTL_BUCKET = "minecraft:axolotl_bucket"
    AXOLOTL_SPAWN_EGG = "minecraft:axolotl_spawn_egg"
    AZALEA = "minecraft:azalea"
    AZALEA_LEAVES = "minecraft:azalea_leaves"
    AZURE_BLUET = "minecraft:azure_bluet"
    BAKED_POTATO = "minecraft:baked_potato"
    BAMBOO_BLOCK = "minecraft:bamboo_block"
    BAMBOO_BUTTON = "minecraft:bamboo_button"
    BAMBOO_CHEST_RAFT = "minecraft:bamboo_chest_raft"
    BAMBOO_DOOR = "minecraft:bamboo_door"
    BAMBOO_FENCE_GATE = "minecraft:bamboo_fence_gate"
    BAMBOO_FENCE = "minecraft:bamboo_fence"
    BAMBOO_HANGING_SIGN = "minecraft:bamboo_hanging_sign"
    BAMBOO = "minecraft:bamboo"
    BAMBOO_MOSAIC = "minecraft:bamboo_mosaic"
    BAMBOO_MOSAIC_SLAB = "minecraft:bamboo_mosaic_slab"
    BAMBOO_MOSAIC_STAIRS = "minecraft:bamboo_mosaic_stairs"
    BAMBOO_PLANKS = "minecraft:bamboo_planks"
    BAMBOO_PRESSURE_PLATE = "minecraft:bamboo_pressure_plate"
    BAMBOO_RAFT = "minecraft:bamboo_raft"
    BAMBOO_SHELF = "minecraft:bamboo_shelf"
    BAMBOO_SIGN = "minecraft:bamboo_sign"
    BAMBOO_SLAB = "minecraft:bamboo_slab"
    BAMBOO_STAIRS = "minecraft:bamboo_stairs"
    BAMBOO_TRAPDOOR = "minecraft:bamboo_trapdoor"
    BARREL = "minecraft:barrel"
    BARRIER = "minecraft:barrier"
    BASALT = "minecraft:basalt"
    BAT_SPAWN_EGG = "minecraft:bat_spawn_egg"
    BEACON = "minecraft:beacon"
    BEDROCK = "minecraft:bedrock"
    BEEF = "minecraft:beef"
    BEEHIVE = "minecraft:beehive"
    BEE_NEST = "minecraft:bee_nest"
    BEE_SPAWN_EGG = "minecraft:bee_spawn_egg"
    BEETROOT = "minecraft:beetroot"
    BEETROOT_SEEDS = "minecraft:beetroot_seeds"
    BEETROOT_SOUP = "minecraft:beetroot_soup"
    BELL = "minecraft:bell"
    BIG_DRIPLEAF = "minecraft:big_dripleaf"
    BIRCH_BOAT = "minecraft:birch_boat"
    BIRCH_BUTTON = "minecraft:birch_button"
    BIRCH_CHEST_BOAT = "minecraft:birch_chest_boat"
    BIRCH_DOOR = "minecraft:birch_door"
    BIRCH_FENCE_GATE = "minecraft:birch_fence_gate"
    BIRCH_FENCE = "minecraft:birch_fence"
    BIRCH_HANGING_SIGN = "minecraft:birch_hanging_sign"
    BIRCH_LEAVES = "minecraft:birch_leaves"
    BIRCH_LOG = "minecraft:birch_log"
    BIRCH_PLANKS = "minecraft:birch_planks"
    BIRCH_PRESSURE_PLATE = "minecraft:birch_pressure_plate"
    BIRCH_SAPLING = "minecraft:birch_sapling"
    BIRCH_SHELF = "minecraft:birch_shelf"
    BIRCH_SIGN = "minecraft:birch_sign"
    BIRCH_SLAB = "minecraft:birch_slab"
    BIRCH_STAIRS = "minecraft:birch_stairs"
    BIRCH_TRAPDOOR = "minecraft:birch_trapdoor"
    BIRCH_WOOD = "minecraft:birch_wood"
    BLACK_BANNER = "minecraft:black_banner"
    BLACK_BED = "minecraft:black_bed"
    BLACK_BUNDLE = "minecraft:black_bundle"
    BLACK_CANDLE = "minecraft:black_candle"
    BLACK_CARPET = "minecraft:black_carpet"
    BLACK_CONCRETE = "minecraft:black_concrete"
    BLACK_CONCRETE_POWDER = "minecraft:black_concrete_powder"
    BLACK_DYE = "minecraft:black_dye"
    BLACK_GLAZED_TERRACOTTA = "minecraft:black_glazed_terracotta"
    BLACK_HARNESS = "minecraft:black_harness"
    BLACK_SHULKER_BOX = "minecraft:black_shulker_box"
    BLACK_STAINED_GLASS = "minecraft:black_stained_glass"
    BLACK_STAINED_GLASS_PANE = "minecraft:black_stained_glass_pane"
    BLACKSTONE = "minecraft:blackstone"
    BLACKSTONE_SLAB = "minecraft:blackstone_slab"
    BLACKSTONE_STAIRS = "minecraft:blackstone_stairs"
    BLACKSTONE_WALL = "minecraft:blackstone_wall"
    BLACK_TERRACOTTA = "minecraft:black_terracotta"
    BLACK_WOOL = "minecraft:black_wool"
    BLADE_POTTERY_SHERD = "minecraft:blade_pottery_sherd"
    BLAST_FURNACE = "minecraft:blast_furnace"
    BLAZE_POWDER = "minecraft:blaze_powder"
    BLAZE_ROD = "minecraft:blaze_rod"
    BLAZE_SPAWN_EGG = "minecraft:blaze_spawn_egg"
    BLUE_BANNER = "minecraft:blue_banner"
    BLUE_BED = "minecraft:blue_bed"
    BLUE_BUNDLE = "minecraft:blue_bundle"
    BLUE_CANDLE = "minecraft:blue_candle"
    BLUE_CARPET = "minecraft:blue_carpet"
    BLUE_CONCRETE = "minecraft:blue_concrete"
    BLUE_CONCRETE_POWDER = "minecraft:blue_concrete_powder"
    BLUE_DYE = "minecraft:blue_dye"
    BLUE_EGG = "minecraft:blue_egg"
    BLUE_GLAZED_TERRACOTTA = "minecraft:blue_glazed_terracotta"
    BLUE_HARNESS = "minecraft:blue_harness"
    BLUE_ICE = "minecraft:blue_ice"
    BLUE_ORCHID = "minecraft:blue_orchid"
    BLUE_SHULKER_BOX = "minecraft:blue_shulker_box"
    BLUE_STAINED_GLASS = "minecraft:blue_stained_glass"
    BLUE_STAINED_GLASS_PANE = "minecraft:blue_stained_glass_pane"
    BLUE_TERRACOTTA = "minecraft:blue_terracotta"
    BLUE_WOOL = "minecraft:blue_wool"
    BOGGED_SPAWN_EGG = "minecraft:bogged_spawn_egg"
    BOLT_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:bolt_armor_trim_smithing_template"
    BONE_BLOCK = "minecraft:bone_block"
    BONE = "minecraft:bone"
    BONE_MEAL = "minecraft:bone_meal"
    BOOK = "minecraft:book"
    BOOKSHELF = "minecraft:bookshelf"
    BORDURE_INDENTED_BANNER_PATTERN = "minecraft:bordure_indented_banner_pattern"
    BOW = "minecraft:bow"
    BOWL = "minecraft:bowl"
    BRAIN_CORAL_BLOCK = "minecraft:brain_coral_block"
    BRAIN_CORAL_FAN = "minecraft:brain_coral_fan"
    BRAIN_CORAL = "minecraft:brain_coral"
    BREAD = "minecraft:bread"
    BREEZE_ROD = "minecraft:breeze_rod"
    BREEZE_SPAWN_EGG = "minecraft:breeze_spawn_egg"
    BREWER_POTTERY_SHERD = "minecraft:brewer_pottery_sherd"
    BREWING_STAND = "minecraft:brewing_stand"
    BRICK = "minecraft:brick"
    BRICKS = "minecraft:bricks"
    BRICK_SLAB = "minecraft:brick_slab"
    BRICK_STAIRS = "minecraft:brick_stairs"
    BRICK_WALL = "minecraft:brick_wall"
    BROWN_BANNER = "minecraft:brown_banner"
    BROWN_BED = "minecraft:brown_bed"
    BROWN_BUNDLE = "minecraft:brown_bundle"
    BROWN_CANDLE = "minecraft:brown_candle"
    BROWN_CARPET = "minecraft:brown_carpet"
    BROWN_CONCRETE = "minecraft:brown_concrete"
    BROWN_CONCRETE_POWDER = "minecraft:brown_concrete_powder"
    BROWN_DYE = "minecraft:brown_dye"
    BROWN_EGG = "minecraft:brown_egg"
    BROWN_GLAZED_TERRACOTTA = "minecraft:brown_glazed_terracotta"
    BROWN_HARNESS = "minecraft:brown_harness"
    BROWN_MUSHROOM_BLOCK = "minecraft:brown_mushroom_block"
    BROWN_MUSHROOM = "minecraft:brown_mushroom"
    BROWN_SHULKER_BOX = "minecraft:brown_shulker_box"
    BROWN_STAINED_GLASS = "minecraft:brown_stained_glass"
    BROWN_STAINED_GLASS_PANE = "minecraft:brown_stained_glass_pane"
    BROWN_TERRACOTTA = "minecraft:brown_terracotta"
    BROWN_WOOL = "minecraft:brown_wool"
    BRUSH = "minecraft:brush"
    BUBBLE_CORAL_BLOCK = "minecraft:bubble_coral_block"
    BUBBLE_CORAL_FAN = "minecraft:bubble_coral_fan"
    BUBBLE_CORAL = "minecraft:bubble_coral"
    BUCKET = "minecraft:bucket"
    BUDDING_AMETHYST = "minecraft:budding_amethyst"
    BUNDLE = "minecraft:bundle"
    BURN_POTTERY_SHERD = "minecraft:burn_pottery_sherd"
    BUSH = "minecraft:bush"
    CACTUS_FLOWER = "minecraft:cactus_flower"
    CACTUS = "minecraft:cactus"
    CAKE = "minecraft:cake"
    CALCITE = "minecraft:calcite"
    CALIBRATED_SCULK_SENSOR = "minecraft:calibrated_sculk_sensor"
    CAMEL_HUSK_SPAWN_EGG = "minecraft:camel_husk_spawn_egg"
    CAMEL_SPAWN_EGG = "minecraft:camel_spawn_egg"
    CAMPFIRE = "minecraft:campfire"
    CANDLE = "minecraft:candle"
    CARROT = "minecraft:carrot"
    CARROT_ON_A_STICK = "minecraft:carrot_on_a_stick"
    CARTOGRAPHY_TABLE = "minecraft:cartography_table"
    CARVED_PUMPKIN = "minecraft:carved_pumpkin"
    CAT_SPAWN_EGG = "minecraft:cat_spawn_egg"
    CAULDRON = "minecraft:cauldron"
    CAVE_SPIDER_SPAWN_EGG = "minecraft:cave_spider_spawn_egg"
    CHAIN_COMMAND_BLOCK = "minecraft:chain_command_block"
    CHAINMAIL_BOOTS = "minecraft:chainmail_boots"
    CHAINMAIL_CHESTPLATE = "minecraft:chainmail_chestplate"
    CHAINMAIL_HELMET = "minecraft:chainmail_helmet"
    CHAINMAIL_LEGGINGS = "minecraft:chainmail_leggings"
    CHARCOAL = "minecraft:charcoal"
    CHERRY_BOAT = "minecraft:cherry_boat"
    CHERRY_BUTTON = "minecraft:cherry_button"
    CHERRY_CHEST_BOAT = "minecraft:cherry_chest_boat"
    CHERRY_DOOR = "minecraft:cherry_door"
    CHERRY_FENCE_GATE = "minecraft:cherry_fence_gate"
    CHERRY_FENCE = "minecraft:cherry_fence"
    CHERRY_HANGING_SIGN = "minecraft:cherry_hanging_sign"
    CHERRY_LEAVES = "minecraft:cherry_leaves"
    CHERRY_LOG = "minecraft:cherry_log"
    CHERRY_PLANKS = "minecraft:cherry_planks"
    CHERRY_PRESSURE_PLATE = "minecraft:cherry_pressure_plate"
    CHERRY_SAPLING = "minecraft:cherry_sapling"
    CHERRY_SHELF = "minecraft:cherry_shelf"
    CHERRY_SIGN = "minecraft:cherry_sign"
    CHERRY_SLAB = "minecraft:cherry_slab"
    CHERRY_STAIRS = "minecraft:cherry_stairs"
    CHERRY_TRAPDOOR = "minecraft:cherry_trapdoor"
    CHERRY_WOOD = "minecraft:cherry_wood"
    CHEST = "minecraft:chest"
    CHEST_MINECART = "minecraft:chest_minecart"
    CHICKEN = "minecraft:chicken"
    CHICKEN_SPAWN_EGG = "minecraft:chicken_spawn_egg"
    CHIPPED_ANVIL = "minecraft:chipped_anvil"
    CHISELED_BOOKSHELF = "minecraft:chiseled_bookshelf"
    CHISELED_COPPER = "minecraft:chiseled_copper"
    CHISELED_DEEPSLATE = "minecraft:chiseled_deepslate"
    CHISELED_NETHER_BRICKS = "minecraft:chiseled_nether_bricks"
    CHISELED_POLISHED_BLACKSTONE = "minecraft:chiseled_polished_blackstone"
    CHISELED_QUARTZ_BLOCK = "minecraft:chiseled_quartz_block"
    CHISELED_RED_SANDSTONE = "minecraft:chiseled_red_sandstone"
    CHISELED_RESIN_BRICKS = "minecraft:chiseled_resin_bricks"
    CHISELED_SANDSTONE = "minecraft:chiseled_sandstone"
    CHISELED_STONE_BRICKS = "minecraft:chiseled_stone_bricks"
    CHISELED_TUFF_BRICKS = "minecraft:chiseled_tuff_bricks"
    CHISELED_TUFF = "minecraft:chiseled_tuff"
    CHORUS_FLOWER = "minecraft:chorus_flower"
    CHORUS_FRUIT = "minecraft:chorus_fruit"
    CHORUS_PLANT = "minecraft:chorus_plant"
    CLAY_BALL = "minecraft:clay_ball"
    CLAY = "minecraft:clay"
    CLOCK = "minecraft:clock"
    CLOSED_EYEBLOSSOM = "minecraft:closed_eyeblossom"
    COAL_BLOCK = "minecraft:coal_block"
    COAL = "minecraft:coal"
    COAL_ORE = "minecraft:coal_ore"
    COARSE_DIRT = "minecraft:coarse_dirt"
    COAST_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:coast_armor_trim_smithing_template"
    COBBLED_DEEPSLATE = "minecraft:cobbled_deepslate"
    COBBLED_DEEPSLATE_SLAB = "minecraft:cobbled_deepslate_slab"
    COBBLED_DEEPSLATE_STAIRS = "minecraft:cobbled_deepslate_stairs"
    COBBLED_DEEPSLATE_WALL = "minecraft:cobbled_deepslate_wall"
    COBBLESTONE = "minecraft:cobblestone"
    COBBLESTONE_SLAB = "minecraft:cobblestone_slab"
    COBBLESTONE_STAIRS = "minecraft:cobblestone_stairs"
    COBBLESTONE_WALL = "minecraft:cobblestone_wall"
    COBWEB = "minecraft:cobweb"
    COCOA_BEANS = "minecraft:cocoa_beans"
    COD_BUCKET = "minecraft:cod_bucket"
    COD = "minecraft:cod"
    COD_SPAWN_EGG = "minecraft:cod_spawn_egg"
    COMMAND_BLOCK = "minecraft:command_block"
    COMMAND_BLOCK_MINECART = "minecraft:command_block_minecart"
    COMPARATOR = "minecraft:comparator"
    COMPASS = "minecraft:compass"
    COMPOSTER = "minecraft:composter"
    CONDUIT = "minecraft:conduit"
    COOKED_BEEF = "minecraft:cooked_beef"
    COOKED_CHICKEN = "minecraft:cooked_chicken"
    COOKED_COD = "minecraft:cooked_cod"
    COOKED_MUTTON = "minecraft:cooked_mutton"
    COOKED_PORKCHOP = "minecraft:cooked_porkchop"
    COOKED_RABBIT = "minecraft:cooked_rabbit"
    COOKED_SALMON = "minecraft:cooked_salmon"
    COOKIE = "minecraft:cookie"
    COPPER_AXE = "minecraft:copper_axe"
    COPPER_BARS = "minecraft:copper_bars"
    COPPER_BLOCK = "minecraft:copper_block"
    COPPER_BOOTS = "minecraft:copper_boots"
    COPPER_BULB = "minecraft:copper_bulb"
    COPPER_CHAIN = "minecraft:copper_chain"
    COPPER_CHEST = "minecraft:copper_chest"
    COPPER_CHESTPLATE = "minecraft:copper_chestplate"
    COPPER_DOOR = "minecraft:copper_door"
    COPPER_GOLEM_SPAWN_EGG = "minecraft:copper_golem_spawn_egg"
    COPPER_GOLEM_STATUE = "minecraft:copper_golem_statue"
    COPPER_GRATE = "minecraft:copper_grate"
    COPPER_HELMET = "minecraft:copper_helmet"
    COPPER_HOE = "minecraft:copper_hoe"
    COPPER_HORSE_ARMOR = "minecraft:copper_horse_armor"
    COPPER_INGOT = "minecraft:copper_ingot"
    COPPER_LANTERN = "minecraft:copper_lantern"
    COPPER_LEGGINGS = "minecraft:copper_leggings"
    COPPER_NAUTILUS_ARMOR = "minecraft:copper_nautilus_armor"
    COPPER_NUGGET = "minecraft:copper_nugget"
    COPPER_ORE = "minecraft:copper_ore"
    COPPER_PICKAXE = "minecraft:copper_pickaxe"
    COPPER_SHOVEL = "minecraft:copper_shovel"
    COPPER_SPEAR = "minecraft:copper_spear"
    COPPER_SWORD = "minecraft:copper_sword"
    COPPER_TORCH = "minecraft:copper_torch"
    COPPER_TRAPDOOR = "minecraft:copper_trapdoor"
    CORNFLOWER = "minecraft:cornflower"
    COW_SPAWN_EGG = "minecraft:cow_spawn_egg"
    CRACKED_DEEPSLATE_BRICKS = "minecraft:cracked_deepslate_bricks"
    CRACKED_DEEPSLATE_TILES = "minecraft:cracked_deepslate_tiles"
    CRACKED_NETHER_BRICKS = "minecraft:cracked_nether_bricks"
    CRACKED_POLISHED_BLACKSTONE_BRICKS = "minecraft:cracked_polished_blackstone_bricks"
    CRACKED_STONE_BRICKS = "minecraft:cracked_stone_bricks"
    CRAFTER = "minecraft:crafter"
    CRAFTING_TABLE = "minecraft:crafting_table"
    CREAKING_HEART = "minecraft:creaking_heart"
    CREAKING_SPAWN_EGG = "minecraft:creaking_spawn_egg"
    CREEPER_BANNER_PATTERN = "minecraft:creeper_banner_pattern"
    CREEPER_HEAD = "minecraft:creeper_head"
    CREEPER_SPAWN_EGG = "minecraft:creeper_spawn_egg"
    CRIMSON_BUTTON = "minecraft:crimson_button"
    CRIMSON_DOOR = "minecraft:crimson_door"
    CRIMSON_FENCE_GATE = "minecraft:crimson_fence_gate"
    CRIMSON_FENCE = "minecraft:crimson_fence"
    CRIMSON_FUNGUS = "minecraft:crimson_fungus"
    CRIMSON_HANGING_SIGN = "minecraft:crimson_hanging_sign"
    CRIMSON_HYPHAE = "minecraft:crimson_hyphae"
    CRIMSON_NYLIUM = "minecraft:crimson_nylium"
    CRIMSON_PLANKS = "minecraft:crimson_planks"
    CRIMSON_PRESSURE_PLATE = "minecraft:crimson_pressure_plate"
    CRIMSON_ROOTS = "minecraft:crimson_roots"
    CRIMSON_SHELF = "minecraft:crimson_shelf"
    CRIMSON_SIGN = "minecraft:crimson_sign"
    CRIMSON_SLAB = "minecraft:crimson_slab"
    CRIMSON_STAIRS = "minecraft:crimson_stairs"
    CRIMSON_STEM = "minecraft:crimson_stem"
    CRIMSON_TRAPDOOR = "minecraft:crimson_trapdoor"
    CROSSBOW = "minecraft:crossbow"
    CRYING_OBSIDIAN = "minecraft:crying_obsidian"
    CUT_COPPER = "minecraft:cut_copper"
    CUT_COPPER_SLAB = "minecraft:cut_copper_slab"
    CUT_COPPER_STAIRS = "minecraft:cut_copper_stairs"
    CUT_RED_SANDSTONE = "minecraft:cut_red_sandstone"
    CUT_RED_SANDSTONE_SLAB = "minecraft:cut_red_sandstone_slab"
    CUT_SANDSTONE = "minecraft:cut_sandstone"
    CUT_SANDSTONE_SLAB = "minecraft:cut_sandstone_slab"
    CYAN_BANNER = "minecraft:cyan_banner"
    CYAN_BED = "minecraft:cyan_bed"
    CYAN_BUNDLE = "minecraft:cyan_bundle"
    CYAN_CANDLE = "minecraft:cyan_candle"
    CYAN_CARPET = "minecraft:cyan_carpet"
    CYAN_CONCRETE = "minecraft:cyan_concrete"
    CYAN_CONCRETE_POWDER = "minecraft:cyan_concrete_powder"
    CYAN_DYE = "minecraft:cyan_dye"
    CYAN_GLAZED_TERRACOTTA = "minecraft:cyan_glazed_terracotta"
    CYAN_HARNESS = "minecraft:cyan_harness"
    CYAN_SHULKER_BOX = "minecraft:cyan_shulker_box"
    CYAN_STAINED_GLASS = "minecraft:cyan_stained_glass"
    CYAN_STAINED_GLASS_PANE = "minecraft:cyan_stained_glass_pane"
    CYAN_TERRACOTTA = "minecraft:cyan_terracotta"
    CYAN_WOOL = "minecraft:cyan_wool"
    DAMAGED_ANVIL = "minecraft:damaged_anvil"
    DANDELION = "minecraft:dandelion"
    DANGER_POTTERY_SHERD = "minecraft:danger_pottery_sherd"
    DARK_OAK_BOAT = "minecraft:dark_oak_boat"
    DARK_OAK_BUTTON = "minecraft:dark_oak_button"
    DARK_OAK_CHEST_BOAT = "minecraft:dark_oak_chest_boat"
    DARK_OAK_DOOR = "minecraft:dark_oak_door"
    DARK_OAK_FENCE_GATE = "minecraft:dark_oak_fence_gate"
    DARK_OAK_FENCE = "minecraft:dark_oak_fence"
    DARK_OAK_HANGING_SIGN = "minecraft:dark_oak_hanging_sign"
    DARK_OAK_LEAVES = "minecraft:dark_oak_leaves"
    DARK_OAK_LOG = "minecraft:dark_oak_log"
    DARK_OAK_PLANKS = "minecraft:dark_oak_planks"
    DARK_OAK_PRESSURE_PLATE = "minecraft:dark_oak_pressure_plate"
    DARK_OAK_SAPLING = "minecraft:dark_oak_sapling"
    DARK_OAK_SHELF = "minecraft:dark_oak_shelf"
    DARK_OAK_SIGN = "minecraft:dark_oak_sign"
    DARK_OAK_SLAB = "minecraft:dark_oak_slab"
    DARK_OAK_STAIRS = "minecraft:dark_oak_stairs"
    DARK_OAK_TRAPDOOR = "minecraft:dark_oak_trapdoor"
    DARK_OAK_WOOD = "minecraft:dark_oak_wood"
    DARK_PRISMARINE = "minecraft:dark_prismarine"
    DARK_PRISMARINE_SLAB = "minecraft:dark_prismarine_slab"
    DARK_PRISMARINE_STAIRS = "minecraft:dark_prismarine_stairs"
    DAYLIGHT_DETECTOR = "minecraft:daylight_detector"
    DEAD_BRAIN_CORAL_BLOCK = "minecraft:dead_brain_coral_block"
    DEAD_BRAIN_CORAL_FAN = "minecraft:dead_brain_coral_fan"
    DEAD_BRAIN_CORAL = "minecraft:dead_brain_coral"
    DEAD_BUBBLE_CORAL_BLOCK = "minecraft:dead_bubble_coral_block"
    DEAD_BUBBLE_CORAL_FAN = "minecraft:dead_bubble_coral_fan"
    DEAD_BUBBLE_CORAL = "minecraft:dead_bubble_coral"
    DEAD_BUSH = "minecraft:dead_bush"
    DEAD_FIRE_CORAL_BLOCK = "minecraft:dead_fire_coral_block"
    DEAD_FIRE_CORAL_FAN = "minecraft:dead_fire_coral_fan"
    DEAD_FIRE_CORAL = "minecraft:dead_fire_coral"
    DEAD_HORN_CORAL_BLOCK = "minecraft:dead_horn_coral_block"
    DEAD_HORN_CORAL_FAN = "minecraft:dead_horn_coral_fan"
    DEAD_HORN_CORAL = "minecraft:dead_horn_coral"
    DEAD_TUBE_CORAL_BLOCK = "minecraft:dead_tube_coral_block"
    DEAD_TUBE_CORAL_FAN = "minecraft:dead_tube_coral_fan"
    DEAD_TUBE_CORAL = "minecraft:dead_tube_coral"
    DEBUG_STICK = "minecraft:debug_stick"
    DECORATED_POT = "minecraft:decorated_pot"
    DEEPSLATE_BRICKS = "minecraft:deepslate_bricks"
    DEEPSLATE_BRICK_SLAB = "minecraft:deepslate_brick_slab"
    DEEPSLATE_BRICK_STAIRS = "minecraft:deepslate_brick_stairs"
    DEEPSLATE_BRICK_WALL = "minecraft:deepslate_brick_wall"
    DEEPSLATE_COAL_ORE = "minecraft:deepslate_coal_ore"
    DEEPSLATE_COPPER_ORE = "minecraft:deepslate_copper_ore"
    DEEPSLATE_DIAMOND_ORE = "minecraft:deepslate_diamond_ore"
    DEEPSLATE_EMERALD_ORE = "minecraft:deepslate_emerald_ore"
    DEEPSLATE_GOLD_ORE = "minecraft:deepslate_gold_ore"
    DEEPSLATE_IRON_ORE = "minecraft:deepslate_iron_ore"
    DEEPSLATE = "minecraft:deepslate"
    DEEPSLATE_LAPIS_ORE = "minecraft:deepslate_lapis_ore"
    DEEPSLATE_REDSTONE_ORE = "minecraft:deepslate_redstone_ore"
    DEEPSLATE_TILES = "minecraft:deepslate_tiles"
    DEEPSLATE_TILE_SLAB = "minecraft:deepslate_tile_slab"
    DEEPSLATE_TILE_STAIRS = "minecraft:deepslate_tile_stairs"
    DEEPSLATE_TILE_WALL = "minecraft:deepslate_tile_wall"
    DETECTOR_RAIL = "minecraft:detector_rail"
    DIAMOND_AXE = "minecraft:diamond_axe"
    DIAMOND_BLOCK = "minecraft:diamond_block"
    DIAMOND_BOOTS = "minecraft:diamond_boots"
    DIAMOND_CHESTPLATE = "minecraft:diamond_chestplate"
    DIAMOND_HELMET = "minecraft:diamond_helmet"
    DIAMOND_HOE = "minecraft:diamond_hoe"
    DIAMOND_HORSE_ARMOR = "minecraft:diamond_horse_armor"
    DIAMOND = "minecraft:diamond"
    DIAMOND_LEGGINGS = "minecraft:diamond_leggings"
    DIAMOND_NAUTILUS_ARMOR = "minecraft:diamond_nautilus_armor"
    DIAMOND_ORE = "minecraft:diamond_ore"
    DIAMOND_PICKAXE = "minecraft:diamond_pickaxe"
    DIAMOND_SHOVEL = "minecraft:diamond_shovel"
    DIAMOND_SPEAR = "minecraft:diamond_spear"
    DIAMOND_SWORD = "minecraft:diamond_sword"
    DIORITE = "minecraft:diorite"
    DIORITE_SLAB = "minecraft:diorite_slab"
    DIORITE_STAIRS = "minecraft:diorite_stairs"
    DIORITE_WALL = "minecraft:diorite_wall"
    DIRT = "minecraft:dirt"
    DIRT_PATH = "minecraft:dirt_path"
    DISC_FRAGMENT_5 = "minecraft:disc_fragment_5"
    DISPENSER = "minecraft:dispenser"
    DOLPHIN_SPAWN_EGG = "minecraft:dolphin_spawn_egg"
    DONKEY_SPAWN_EGG = "minecraft:donkey_spawn_egg"
    DRAGON_BREATH = "minecraft:dragon_breath"
    DRAGON_EGG = "minecraft:dragon_egg"
    DRAGON_HEAD = "minecraft:dragon_head"
    DRIED_GHAST = "minecraft:dried_ghast"
    DRIED_KELP_BLOCK = "minecraft:dried_kelp_block"
    DRIED_KELP = "minecraft:dried_kelp"
    DRIPSTONE_BLOCK = "minecraft:dripstone_block"
    DROPPER = "minecraft:dropper"
    DROWNED_SPAWN_EGG = "minecraft:drowned_spawn_egg"
    DUNE_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:dune_armor_trim_smithing_template"
    ECHO_SHARD = "minecraft:echo_shard"
    EGG = "minecraft:egg"
    ELDER_GUARDIAN_SPAWN_EGG = "minecraft:elder_guardian_spawn_egg"
    ELYTRA = "minecraft:elytra"
    EMERALD_BLOCK = "minecraft:emerald_block"
    EMERALD = "minecraft:emerald"
    EMERALD_ORE = "minecraft:emerald_ore"
    ENCHANTED_BOOK = "minecraft:enchanted_book"
    ENCHANTED_GOLDEN_APPLE = "minecraft:enchanted_golden_apple"
    ENCHANTING_TABLE = "minecraft:enchanting_table"
    END_CRYSTAL = "minecraft:end_crystal"
    ENDER_CHEST = "minecraft:ender_chest"
    ENDER_DRAGON_SPAWN_EGG = "minecraft:ender_dragon_spawn_egg"
    ENDER_EYE = "minecraft:ender_eye"
    ENDERMAN_SPAWN_EGG = "minecraft:enderman_spawn_egg"
    ENDERMITE_SPAWN_EGG = "minecraft:endermite_spawn_egg"
    ENDER_PEARL = "minecraft:ender_pearl"
    END_PORTAL_FRAME = "minecraft:end_portal_frame"
    END_ROD = "minecraft:end_rod"
    END_STONE_BRICKS = "minecraft:end_stone_bricks"
    END_STONE_BRICK_SLAB = "minecraft:end_stone_brick_slab"
    END_STONE_BRICK_STAIRS = "minecraft:end_stone_brick_stairs"
    END_STONE_BRICK_WALL = "minecraft:end_stone_brick_wall"
    END_STONE = "minecraft:end_stone"
    EVOKER_SPAWN_EGG = "minecraft:evoker_spawn_egg"
    EXPERIENCE_BOTTLE = "minecraft:experience_bottle"
    EXPLORER_POTTERY_SHERD = "minecraft:explorer_pottery_sherd"
    EXPOSED_CHISELED_COPPER = "minecraft:exposed_chiseled_copper"
    EXPOSED_COPPER_BARS = "minecraft:exposed_copper_bars"
    EXPOSED_COPPER_BULB = "minecraft:exposed_copper_bulb"
    EXPOSED_COPPER_CHAIN = "minecraft:exposed_copper_chain"
    EXPOSED_COPPER_CHEST = "minecraft:exposed_copper_chest"
    EXPOSED_COPPER_DOOR = "minecraft:exposed_copper_door"
    EXPOSED_COPPER_GOLEM_STATUE = "minecraft:exposed_copper_golem_statue"
    EXPOSED_COPPER_GRATE = "minecraft:exposed_copper_grate"
    EXPOSED_COPPER = "minecraft:exposed_copper"
    EXPOSED_COPPER_LANTERN = "minecraft:exposed_copper_lantern"
    EXPOSED_COPPER_TRAPDOOR = "minecraft:exposed_copper_trapdoor"
    EXPOSED_CUT_COPPER = "minecraft:exposed_cut_copper"
    EXPOSED_CUT_COPPER_SLAB = "minecraft:exposed_cut_copper_slab"
    EXPOSED_CUT_COPPER_STAIRS = "minecraft:exposed_cut_copper_stairs"
    EXPOSED_LIGHTNING_ROD = "minecraft:exposed_lightning_rod"
    EYE_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:eye_armor_trim_smithing_template"
    FARMLAND = "minecraft:farmland"
    FEATHER = "minecraft:feather"
    FERMENTED_SPIDER_EYE = "minecraft:fermented_spider_eye"
    FERN = "minecraft:fern"
    FIELD_MASONED_BANNER_PATTERN = "minecraft:field_masoned_banner_pattern"
    FILLED_MAP = "minecraft:filled_map"
    FIRE_CHARGE = "minecraft:fire_charge"
    FIRE_CORAL_BLOCK = "minecraft:fire_coral_block"
    FIRE_CORAL_FAN = "minecraft:fire_coral_fan"
    FIRE_CORAL = "minecraft:fire_coral"
    FIREFLY_BUSH = "minecraft:firefly_bush"
    FIREWORK_ROCKET = "minecraft:firework_rocket"
    FIREWORK_STAR = "minecraft:firework_star"
    FISHING_ROD = "minecraft:fishing_rod"
    FLETCHING_TABLE = "minecraft:fletching_table"
    FLINT_AND_STEEL = "minecraft:flint_and_steel"
    FLINT = "minecraft:flint"
    FLOW_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:flow_armor_trim_smithing_template"
    FLOW_BANNER_PATTERN = "minecraft:flow_banner_pattern"
    FLOWER_BANNER_PATTERN = "minecraft:flower_banner_pattern"
    FLOWERING_AZALEA = "minecraft:flowering_azalea"
    FLOWERING_AZALEA_LEAVES = "minecraft:flowering_azalea_leaves"
    FLOWER_POT = "minecraft:flower_pot"
    FLOW_POTTERY_SHERD = "minecraft:flow_pottery_sherd"
    FOX_SPAWN_EGG = "minecraft:fox_spawn_egg"
    FRIEND_POTTERY_SHERD = "minecraft:friend_pottery_sherd"
    FROG_SPAWN_EGG = "minecraft:frog_spawn_egg"
    FROGSPAWN = "minecraft:frogspawn"
    FURNACE = "minecraft:furnace"
    FURNACE_MINECART = "minecraft:furnace_minecart"
    GHAST_SPAWN_EGG = "minecraft:ghast_spawn_egg"
    GHAST_TEAR = "minecraft:ghast_tear"
    GILDED_BLACKSTONE = "minecraft:gilded_blackstone"
    GLASS_BOTTLE = "minecraft:glass_bottle"
    GLASS = "minecraft:glass"
    GLASS_PANE = "minecraft:glass_pane"
    GLISTERING_MELON_SLICE = "minecraft:glistering_melon_slice"
    GLOBE_BANNER_PATTERN = "minecraft:globe_banner_pattern"
    GLOW_BERRIES = "minecraft:glow_berries"
    GLOW_INK_SAC = "minecraft:glow_ink_sac"
    GLOW_ITEM_FRAME = "minecraft:glow_item_frame"
    GLOW_LICHEN = "minecraft:glow_lichen"
    GLOW_SQUID_SPAWN_EGG = "minecraft:glow_squid_spawn_egg"
    GLOWSTONE_DUST = "minecraft:glowstone_dust"
    GLOWSTONE = "minecraft:glowstone"
    GOAT_HORN = "minecraft:goat_horn"
    GOAT_SPAWN_EGG = "minecraft:goat_spawn_egg"
    GOLD_BLOCK = "minecraft:gold_block"
    GOLDEN_APPLE = "minecraft:golden_apple"
    GOLDEN_AXE = "minecraft:golden_axe"
    GOLDEN_BOOTS = "minecraft:golden_boots"
    GOLDEN_CARROT = "minecraft:golden_carrot"
    GOLDEN_CHESTPLATE = "minecraft:golden_chestplate"
    GOLDEN_HELMET = "minecraft:golden_helmet"
    GOLDEN_HOE = "minecraft:golden_hoe"
    GOLDEN_HORSE_ARMOR = "minecraft:golden_horse_armor"
    GOLDEN_LEGGINGS = "minecraft:golden_leggings"
    GOLDEN_NAUTILUS_ARMOR = "minecraft:golden_nautilus_armor"
    GOLDEN_PICKAXE = "minecraft:golden_pickaxe"
    GOLDEN_SHOVEL = "minecraft:golden_shovel"
    GOLDEN_SPEAR = "minecraft:golden_spear"
    GOLDEN_SWORD = "minecraft:golden_sword"
    GOLD_INGOT = "minecraft:gold_ingot"
    GOLD_NUGGET = "minecraft:gold_nugget"
    GOLD_ORE = "minecraft:gold_ore"
    GRANITE = "minecraft:granite"
    GRANITE_SLAB = "minecraft:granite_slab"
    GRANITE_STAIRS = "minecraft:granite_stairs"
    GRANITE_WALL = "minecraft:granite_wall"
    GRASS_BLOCK = "minecraft:grass_block"
    GRAVEL = "minecraft:gravel"
    GRAY_BANNER = "minecraft:gray_banner"
    GRAY_BED = "minecraft:gray_bed"
    GRAY_BUNDLE = "minecraft:gray_bundle"
    GRAY_CANDLE = "minecraft:gray_candle"
    GRAY_CARPET = "minecraft:gray_carpet"
    GRAY_CONCRETE = "minecraft:gray_concrete"
    GRAY_CONCRETE_POWDER = "minecraft:gray_concrete_powder"
    GRAY_DYE = "minecraft:gray_dye"
    GRAY_GLAZED_TERRACOTTA = "minecraft:gray_glazed_terracotta"
    GRAY_HARNESS = "minecraft:gray_harness"
    GRAY_SHULKER_BOX = "minecraft:gray_shulker_box"
    GRAY_STAINED_GLASS = "minecraft:gray_stained_glass"
    GRAY_STAINED_GLASS_PANE = "minecraft:gray_stained_glass_pane"
    GRAY_TERRACOTTA = "minecraft:gray_terracotta"
    GRAY_WOOL = "minecraft:gray_wool"
    GREEN_BANNER = "minecraft:green_banner"
    GREEN_BED = "minecraft:green_bed"
    GREEN_BUNDLE = "minecraft:green_bundle"
    GREEN_CANDLE = "minecraft:green_candle"
    GREEN_CARPET = "minecraft:green_carpet"
    GREEN_CONCRETE = "minecraft:green_concrete"
    GREEN_CONCRETE_POWDER = "minecraft:green_concrete_powder"
    GREEN_DYE = "minecraft:green_dye"
    GREEN_GLAZED_TERRACOTTA = "minecraft:green_glazed_terracotta"
    GREEN_HARNESS = "minecraft:green_harness"
    GREEN_SHULKER_BOX = "minecraft:green_shulker_box"
    GREEN_STAINED_GLASS = "minecraft:green_stained_glass"
    GREEN_STAINED_GLASS_PANE = "minecraft:green_stained_glass_pane"
    GREEN_TERRACOTTA = "minecraft:green_terracotta"
    GREEN_WOOL = "minecraft:green_wool"
    GRINDSTONE = "minecraft:grindstone"
    GUARDIAN_SPAWN_EGG = "minecraft:guardian_spawn_egg"
    GUNPOWDER = "minecraft:gunpowder"
    GUSTER_BANNER_PATTERN = "minecraft:guster_banner_pattern"
    GUSTER_POTTERY_SHERD = "minecraft:guster_pottery_sherd"
    HANGING_ROOTS = "minecraft:hanging_roots"
    HAPPY_GHAST_SPAWN_EGG = "minecraft:happy_ghast_spawn_egg"
    HAY_BLOCK = "minecraft:hay_block"
    HEARTBREAK_POTTERY_SHERD = "minecraft:heartbreak_pottery_sherd"
    HEART_OF_THE_SEA = "minecraft:heart_of_the_sea"
    HEART_POTTERY_SHERD = "minecraft:heart_pottery_sherd"
    HEAVY_CORE = "minecraft:heavy_core"
    HEAVY_WEIGHTED_PRESSURE_PLATE = "minecraft:heavy_weighted_pressure_plate"
    HOGLIN_SPAWN_EGG = "minecraft:hoglin_spawn_egg"
    HONEY_BLOCK = "minecraft:honey_block"
    HONEY_BOTTLE = "minecraft:honey_bottle"
    HONEYCOMB_BLOCK = "minecraft:honeycomb_block"
    HONEYCOMB = "minecraft:honeycomb"
    HOPPER = "minecraft:hopper"
    HOPPER_MINECART = "minecraft:hopper_minecart"
    HORN_CORAL_BLOCK = "minecraft:horn_coral_block"
    HORN_CORAL_FAN = "minecraft:horn_coral_fan"
    HORN_CORAL = "minecraft:horn_coral"
    HORSE_SPAWN_EGG = "minecraft:horse_spawn_egg"
    HOST_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:host_armor_trim_smithing_template"
    HOWL_POTTERY_SHERD = "minecraft:howl_pottery_sherd"
    HUSK_SPAWN_EGG = "minecraft:husk_spawn_egg"
    ICE = "minecraft:ice"
    INFESTED_CHISELED_STONE_BRICKS = "minecraft:infested_chiseled_stone_bricks"
    INFESTED_COBBLESTONE = "minecraft:infested_cobblestone"
    INFESTED_CRACKED_STONE_BRICKS = "minecraft:infested_cracked_stone_bricks"
    INFESTED_DEEPSLATE = "minecraft:infested_deepslate"
    INFESTED_MOSSY_STONE_BRICKS = "minecraft:infested_mossy_stone_bricks"
    INFESTED_STONE_BRICKS = "minecraft:infested_stone_bricks"
    INFESTED_STONE = "minecraft:infested_stone"
    INK_SAC = "minecraft:ink_sac"
    IRON_AXE = "minecraft:iron_axe"
    IRON_BARS = "minecraft:iron_bars"
    IRON_BLOCK = "minecraft:iron_block"
    IRON_BOOTS = "minecraft:iron_boots"
    IRON_CHAIN = "minecraft:iron_chain"
    IRON_CHESTPLATE = "minecraft:iron_chestplate"
    IRON_DOOR = "minecraft:iron_door"
    IRON_GOLEM_SPAWN_EGG = "minecraft:iron_golem_spawn_egg"
    IRON_HELMET = "minecraft:iron_helmet"
    IRON_HOE = "minecraft:iron_hoe"
    IRON_HORSE_ARMOR = "minecraft:iron_horse_armor"
    IRON_INGOT = "minecraft:iron_ingot"
    IRON_LEGGINGS = "minecraft:iron_leggings"
    IRON_NAUTILUS_ARMOR = "minecraft:iron_nautilus_armor"
    IRON_NUGGET = "minecraft:iron_nugget"
    IRON_ORE = "minecraft:iron_ore"
    IRON_PICKAXE = "minecraft:iron_pickaxe"
    IRON_SHOVEL = "minecraft:iron_shovel"
    IRON_SPEAR = "minecraft:iron_spear"
    IRON_SWORD = "minecraft:iron_sword"
    IRON_TRAPDOOR = "minecraft:iron_trapdoor"
    ITEM_FRAME = "minecraft:item_frame"
    JACK_O_LANTERN = "minecraft:jack_o_lantern"
    JIGSAW = "minecraft:jigsaw"
    JUKEBOX = "minecraft:jukebox"
    JUNGLE_BOAT = "minecraft:jungle_boat"
    JUNGLE_BUTTON = "minecraft:jungle_button"
    JUNGLE_CHEST_BOAT = "minecraft:jungle_chest_boat"
    JUNGLE_DOOR = "minecraft:jungle_door"
    JUNGLE_FENCE_GATE = "minecraft:jungle_fence_gate"
    JUNGLE_FENCE = "minecraft:jungle_fence"
    JUNGLE_HANGING_SIGN = "minecraft:jungle_hanging_sign"
    JUNGLE_LEAVES = "minecraft:jungle_leaves"
    JUNGLE_LOG = "minecraft:jungle_log"
    JUNGLE_PLANKS = "minecraft:jungle_planks"
    JUNGLE_PRESSURE_PLATE = "minecraft:jungle_pressure_plate"
    JUNGLE_SAPLING = "minecraft:jungle_sapling"
    JUNGLE_SHELF = "minecraft:jungle_shelf"
    JUNGLE_SIGN = "minecraft:jungle_sign"
    JUNGLE_SLAB = "minecraft:jungle_slab"
    JUNGLE_STAIRS = "minecraft:jungle_stairs"
    JUNGLE_TRAPDOOR = "minecraft:jungle_trapdoor"
    JUNGLE_WOOD = "minecraft:jungle_wood"
    KELP = "minecraft:kelp"
    KNOWLEDGE_BOOK = "minecraft:knowledge_book"
    LADDER = "minecraft:ladder"
    LANTERN = "minecraft:lantern"
    LAPIS_BLOCK = "minecraft:lapis_block"
    LAPIS_LAZULI = "minecraft:lapis_lazuli"
    LAPIS_ORE = "minecraft:lapis_ore"
    LARGE_AMETHYST_BUD = "minecraft:large_amethyst_bud"
    LARGE_FERN = "minecraft:large_fern"
    LAVA_BUCKET = "minecraft:lava_bucket"
    LEAD = "minecraft:lead"
    LEAF_LITTER = "minecraft:leaf_litter"
    LEATHER_BOOTS = "minecraft:leather_boots"
    LEATHER_CHESTPLATE = "minecraft:leather_chestplate"
    LEATHER_HELMET = "minecraft:leather_helmet"
    LEATHER_HORSE_ARMOR = "minecraft:leather_horse_armor"
    LEATHER = "minecraft:leather"
    LEATHER_LEGGINGS = "minecraft:leather_leggings"
    LECTERN = "minecraft:lectern"
    LEVER = "minecraft:lever"
    LIGHT_BLUE_BANNER = "minecraft:light_blue_banner"
    LIGHT_BLUE_BED = "minecraft:light_blue_bed"
    LIGHT_BLUE_BUNDLE = "minecraft:light_blue_bundle"
    LIGHT_BLUE_CANDLE = "minecraft:light_blue_candle"
    LIGHT_BLUE_CARPET = "minecraft:light_blue_carpet"
    LIGHT_BLUE_CONCRETE = "minecraft:light_blue_concrete"
    LIGHT_BLUE_CONCRETE_POWDER = "minecraft:light_blue_concrete_powder"
    LIGHT_BLUE_DYE = "minecraft:light_blue_dye"
    LIGHT_BLUE_GLAZED_TERRACOTTA = "minecraft:light_blue_glazed_terracotta"
    LIGHT_BLUE_HARNESS = "minecraft:light_blue_harness"
    LIGHT_BLUE_SHULKER_BOX = "minecraft:light_blue_shulker_box"
    LIGHT_BLUE_STAINED_GLASS = "minecraft:light_blue_stained_glass"
    LIGHT_BLUE_STAINED_GLASS_PANE = "minecraft:light_blue_stained_glass_pane"
    LIGHT_BLUE_TERRACOTTA = "minecraft:light_blue_terracotta"
    LIGHT_BLUE_WOOL = "minecraft:light_blue_wool"
    LIGHT_GRAY_BANNER = "minecraft:light_gray_banner"
    LIGHT_GRAY_BED = "minecraft:light_gray_bed"
    LIGHT_GRAY_BUNDLE = "minecraft:light_gray_bundle"
    LIGHT_GRAY_CANDLE = "minecraft:light_gray_candle"
    LIGHT_GRAY_CARPET = "minecraft:light_gray_carpet"
    LIGHT_GRAY_CONCRETE = "minecraft:light_gray_concrete"
    LIGHT_GRAY_CONCRETE_POWDER = "minecraft:light_gray_concrete_powder"
    LIGHT_GRAY_DYE = "minecraft:light_gray_dye"
    LIGHT_GRAY_GLAZED_TERRACOTTA = "minecraft:light_gray_glazed_terracotta"
    LIGHT_GRAY_HARNESS = "minecraft:light_gray_harness"
    LIGHT_GRAY_SHULKER_BOX = "minecraft:light_gray_shulker_box"
    LIGHT_GRAY_STAINED_GLASS = "minecraft:light_gray_stained_glass"
    LIGHT_GRAY_STAINED_GLASS_PANE = "minecraft:light_gray_stained_glass_pane"
    LIGHT_GRAY_TERRACOTTA = "minecraft:light_gray_terracotta"
    LIGHT_GRAY_WOOL = "minecraft:light_gray_wool"
    LIGHT = "minecraft:light"
    LIGHTNING_ROD = "minecraft:lightning_rod"
    LIGHT_WEIGHTED_PRESSURE_PLATE = "minecraft:light_weighted_pressure_plate"
    LILAC = "minecraft:lilac"
    LILY_OF_THE_VALLEY = "minecraft:lily_of_the_valley"
    LILY_PAD = "minecraft:lily_pad"
    LIME_BANNER = "minecraft:lime_banner"
    LIME_BED = "minecraft:lime_bed"
    LIME_BUNDLE = "minecraft:lime_bundle"
    LIME_CANDLE = "minecraft:lime_candle"
    LIME_CARPET = "minecraft:lime_carpet"
    LIME_CONCRETE = "minecraft:lime_concrete"
    LIME_CONCRETE_POWDER = "minecraft:lime_concrete_powder"
    LIME_DYE = "minecraft:lime_dye"
    LIME_GLAZED_TERRACOTTA = "minecraft:lime_glazed_terracotta"
    LIME_HARNESS = "minecraft:lime_harness"
    LIME_SHULKER_BOX = "minecraft:lime_shulker_box"
    LIME_STAINED_GLASS = "minecraft:lime_stained_glass"
    LIME_STAINED_GLASS_PANE = "minecraft:lime_stained_glass_pane"
    LIME_TERRACOTTA = "minecraft:lime_terracotta"
    LIME_WOOL = "minecraft:lime_wool"
    LINGERING_POTION = "minecraft:lingering_potion"
    LLAMA_SPAWN_EGG = "minecraft:llama_spawn_egg"
    LODESTONE = "minecraft:lodestone"
    LOOM = "minecraft:loom"
    MACE = "minecraft:mace"
    MAGENTA_BANNER = "minecraft:magenta_banner"
    MAGENTA_BED = "minecraft:magenta_bed"
    MAGENTA_BUNDLE = "minecraft:magenta_bundle"
    MAGENTA_CANDLE = "minecraft:magenta_candle"
    MAGENTA_CARPET = "minecraft:magenta_carpet"
    MAGENTA_CONCRETE = "minecraft:magenta_concrete"
    MAGENTA_CONCRETE_POWDER = "minecraft:magenta_concrete_powder"
    MAGENTA_DYE = "minecraft:magenta_dye"
    MAGENTA_GLAZED_TERRACOTTA = "minecraft:magenta_glazed_terracotta"
    MAGENTA_HARNESS = "minecraft:magenta_harness"
    MAGENTA_SHULKER_BOX = "minecraft:magenta_shulker_box"
    MAGENTA_STAINED_GLASS = "minecraft:magenta_stained_glass"
    MAGENTA_STAINED_GLASS_PANE = "minecraft:magenta_stained_glass_pane"
    MAGENTA_TERRACOTTA = "minecraft:magenta_terracotta"
    MAGENTA_WOOL = "minecraft:magenta_wool"
    MAGMA_BLOCK = "minecraft:magma_block"
    MAGMA_CREAM = "minecraft:magma_cream"
    MAGMA_CUBE_SPAWN_EGG = "minecraft:magma_cube_spawn_egg"
    MANGROVE_BOAT = "minecraft:mangrove_boat"
    MANGROVE_BUTTON = "minecraft:mangrove_button"
    MANGROVE_CHEST_BOAT = "minecraft:mangrove_chest_boat"
    MANGROVE_DOOR = "minecraft:mangrove_door"
    MANGROVE_FENCE_GATE = "minecraft:mangrove_fence_gate"
    MANGROVE_FENCE = "minecraft:mangrove_fence"
    MANGROVE_HANGING_SIGN = "minecraft:mangrove_hanging_sign"
    MANGROVE_LEAVES = "minecraft:mangrove_leaves"
    MANGROVE_LOG = "minecraft:mangrove_log"
    MANGROVE_PLANKS = "minecraft:mangrove_planks"
    MANGROVE_PRESSURE_PLATE = "minecraft:mangrove_pressure_plate"
    MANGROVE_PROPAGULE = "minecraft:mangrove_propagule"
    MANGROVE_ROOTS = "minecraft:mangrove_roots"
    MANGROVE_SHELF = "minecraft:mangrove_shelf"
    MANGROVE_SIGN = "minecraft:mangrove_sign"
    MANGROVE_SLAB = "minecraft:mangrove_slab"
    MANGROVE_STAIRS = "minecraft:mangrove_stairs"
    MANGROVE_TRAPDOOR = "minecraft:mangrove_trapdoor"
    MANGROVE_WOOD = "minecraft:mangrove_wood"
    MAP = "minecraft:map"
    MEDIUM_AMETHYST_BUD = "minecraft:medium_amethyst_bud"
    MELON = "minecraft:melon"
    MELON_SEEDS = "minecraft:melon_seeds"
    MELON_SLICE = "minecraft:melon_slice"
    MILK_BUCKET = "minecraft:milk_bucket"
    MINECART = "minecraft:minecart"
    MINER_POTTERY_SHERD = "minecraft:miner_pottery_sherd"
    MOJANG_BANNER_PATTERN = "minecraft:mojang_banner_pattern"
    MOOSHROOM_SPAWN_EGG = "minecraft:mooshroom_spawn_egg"
    MOSS_BLOCK = "minecraft:moss_block"
    MOSS_CARPET = "minecraft:moss_carpet"
    MOSSY_COBBLESTONE = "minecraft:mossy_cobblestone"
    MOSSY_COBBLESTONE_SLAB = "minecraft:mossy_cobblestone_slab"
    MOSSY_COBBLESTONE_STAIRS = "minecraft:mossy_cobblestone_stairs"
    MOSSY_COBBLESTONE_WALL = "minecraft:mossy_cobblestone_wall"
    MOSSY_STONE_BRICKS = "minecraft:mossy_stone_bricks"
    MOSSY_STONE_BRICK_SLAB = "minecraft:mossy_stone_brick_slab"
    MOSSY_STONE_BRICK_STAIRS = "minecraft:mossy_stone_brick_stairs"
    MOSSY_STONE_BRICK_WALL = "minecraft:mossy_stone_brick_wall"
    MOURNER_POTTERY_SHERD = "minecraft:mourner_pottery_sherd"
    MUD_BRICKS = "minecraft:mud_bricks"
    MUD_BRICK_SLAB = "minecraft:mud_brick_slab"
    MUD_BRICK_STAIRS = "minecraft:mud_brick_stairs"
    MUD_BRICK_WALL = "minecraft:mud_brick_wall"
    MUDDY_MANGROVE_ROOTS = "minecraft:muddy_mangrove_roots"
    MUD = "minecraft:mud"
    MULE_SPAWN_EGG = "minecraft:mule_spawn_egg"
    MUSHROOM_STEM = "minecraft:mushroom_stem"
    MUSHROOM_STEW = "minecraft:mushroom_stew"
    MUSIC_DISC_11 = "minecraft:music_disc_11"
    MUSIC_DISC_13 = "minecraft:music_disc_13"
    MUSIC_DISC_5 = "minecraft:music_disc_5"
    MUSIC_DISC_BLOCKS = "minecraft:music_disc_blocks"
    MUSIC_DISC_CAT = "minecraft:music_disc_cat"
    MUSIC_DISC_CHIRP = "minecraft:music_disc_chirp"
    MUSIC_DISC_CREATOR = "minecraft:music_disc_creator"
    MUSIC_DISC_CREATOR_MUSIC_BOX = "minecraft:music_disc_creator_music_box"
    MUSIC_DISC_FAR = "minecraft:music_disc_far"
    MUSIC_DISC_LAVA_CHICKEN = "minecraft:music_disc_lava_chicken"
    MUSIC_DISC_MALL = "minecraft:music_disc_mall"
    MUSIC_DISC_MELLOHI = "minecraft:music_disc_mellohi"
    MUSIC_DISC_OTHERSIDE = "minecraft:music_disc_otherside"
    MUSIC_DISC_PIGSTEP = "minecraft:music_disc_pigstep"
    MUSIC_DISC_PRECIPICE = "minecraft:music_disc_precipice"
    MUSIC_DISC_RELIC = "minecraft:music_disc_relic"
    MUSIC_DISC_STAL = "minecraft:music_disc_stal"
    MUSIC_DISC_STRAD = "minecraft:music_disc_strad"
    MUSIC_DISC_TEARS = "minecraft:music_disc_tears"
    MUSIC_DISC_WAIT = "minecraft:music_disc_wait"
    MUSIC_DISC_WARD = "minecraft:music_disc_ward"
    MUTTON = "minecraft:mutton"
    MYCELIUM = "minecraft:mycelium"
    NAME_TAG = "minecraft:name_tag"
    NAUTILUS_SHELL = "minecraft:nautilus_shell"
    NAUTILUS_SPAWN_EGG = "minecraft:nautilus_spawn_egg"
    NETHER_BRICK_FENCE = "minecraft:nether_brick_fence"
    NETHER_BRICK = "minecraft:nether_brick"
    NETHER_BRICKS = "minecraft:nether_bricks"
    NETHER_BRICK_SLAB = "minecraft:nether_brick_slab"
    NETHER_BRICK_STAIRS = "minecraft:nether_brick_stairs"
    NETHER_BRICK_WALL = "minecraft:nether_brick_wall"
    NETHER_GOLD_ORE = "minecraft:nether_gold_ore"
    NETHERITE_AXE = "minecraft:netherite_axe"
    NETHERITE_BLOCK = "minecraft:netherite_block"
    NETHERITE_BOOTS = "minecraft:netherite_boots"
    NETHERITE_CHESTPLATE = "minecraft:netherite_chestplate"
    NETHERITE_HELMET = "minecraft:netherite_helmet"
    NETHERITE_HOE = "minecraft:netherite_hoe"
    NETHERITE_HORSE_ARMOR = "minecraft:netherite_horse_armor"
    NETHERITE_INGOT = "minecraft:netherite_ingot"
    NETHERITE_LEGGINGS = "minecraft:netherite_leggings"
    NETHERITE_NAUTILUS_ARMOR = "minecraft:netherite_nautilus_armor"
    NETHERITE_PICKAXE = "minecraft:netherite_pickaxe"
    NETHERITE_SCRAP = "minecraft:netherite_scrap"
    NETHERITE_SHOVEL = "minecraft:netherite_shovel"
    NETHERITE_SPEAR = "minecraft:netherite_spear"
    NETHERITE_SWORD = "minecraft:netherite_sword"
    NETHERITE_UPGRADE_SMITHING_TEMPLATE = "minecraft:netherite_upgrade_smithing_template"
    NETHER_QUARTZ_ORE = "minecraft:nether_quartz_ore"
    NETHERRACK = "minecraft:netherrack"
    NETHER_SPROUTS = "minecraft:nether_sprouts"
    NETHER_STAR = "minecraft:nether_star"
    NETHER_WART_BLOCK = "minecraft:nether_wart_block"
    NETHER_WART = "minecraft:nether_wart"
    NOTE_BLOCK = "minecraft:note_block"
    OAK_BOAT = "minecraft:oak_boat"
    OAK_BUTTON = "minecraft:oak_button"
    OAK_CHEST_BOAT = "minecraft:oak_chest_boat"
    OAK_DOOR = "minecraft:oak_door"
    OAK_FENCE_GATE = "minecraft:oak_fence_gate"
    OAK_FENCE = "minecraft:oak_fence"
    OAK_HANGING_SIGN = "minecraft:oak_hanging_sign"
    OAK_LEAVES = "minecraft:oak_leaves"
    OAK_LOG = "minecraft:oak_log"
    OAK_PLANKS = "minecraft:oak_planks"
    OAK_PRESSURE_PLATE = "minecraft:oak_pressure_plate"
    OAK_SAPLING = "minecraft:oak_sapling"
    OAK_SHELF = "minecraft:oak_shelf"
    OAK_SIGN = "minecraft:oak_sign"
    OAK_SLAB = "minecraft:oak_slab"
    OAK_STAIRS = "minecraft:oak_stairs"
    OAK_TRAPDOOR = "minecraft:oak_trapdoor"
    OAK_WOOD = "minecraft:oak_wood"
    OBSERVER = "minecraft:observer"
    OBSIDIAN = "minecraft:obsidian"
    OCELOT_SPAWN_EGG = "minecraft:ocelot_spawn_egg"
    OCHRE_FROGLIGHT = "minecraft:ochre_froglight"
    OMINOUS_BOTTLE = "minecraft:ominous_bottle"
    OMINOUS_TRIAL_KEY = "minecraft:ominous_trial_key"
    OPEN_EYEBLOSSOM = "minecraft:open_eyeblossom"
    ORANGE_BANNER = "minecraft:orange_banner"
    ORANGE_BED = "minecraft:orange_bed"
    ORANGE_BUNDLE = "minecraft:orange_bundle"
    ORANGE_CANDLE = "minecraft:orange_candle"
    ORANGE_CARPET = "minecraft:orange_carpet"
    ORANGE_CONCRETE = "minecraft:orange_concrete"
    ORANGE_CONCRETE_POWDER = "minecraft:orange_concrete_powder"
    ORANGE_DYE = "minecraft:orange_dye"
    ORANGE_GLAZED_TERRACOTTA = "minecraft:orange_glazed_terracotta"
    ORANGE_HARNESS = "minecraft:orange_harness"
    ORANGE_SHULKER_BOX = "minecraft:orange_shulker_box"
    ORANGE_STAINED_GLASS = "minecraft:orange_stained_glass"
    ORANGE_STAINED_GLASS_PANE = "minecraft:orange_stained_glass_pane"
    ORANGE_TERRACOTTA = "minecraft:orange_terracotta"
    ORANGE_TULIP = "minecraft:orange_tulip"
    ORANGE_WOOL = "minecraft:orange_wool"
    OXEYE_DAISY = "minecraft:oxeye_daisy"
    OXIDIZED_CHISELED_COPPER = "minecraft:oxidized_chiseled_copper"
    OXIDIZED_COPPER_BARS = "minecraft:oxidized_copper_bars"
    OXIDIZED_COPPER_BULB = "minecraft:oxidized_copper_bulb"
    OXIDIZED_COPPER_CHAIN = "minecraft:oxidized_copper_chain"
    OXIDIZED_COPPER_CHEST = "minecraft:oxidized_copper_chest"
    OXIDIZED_COPPER_DOOR = "minecraft:oxidized_copper_door"
    OXIDIZED_COPPER_GOLEM_STATUE = "minecraft:oxidized_copper_golem_statue"
    OXIDIZED_COPPER_GRATE = "minecraft:oxidized_copper_grate"
    OXIDIZED_COPPER = "minecraft:oxidized_copper"
    OXIDIZED_COPPER_LANTERN = "minecraft:oxidized_copper_lantern"
    OXIDIZED_COPPER_TRAPDOOR = "minecraft:oxidized_copper_trapdoor"
    OXIDIZED_CUT_COPPER = "minecraft:oxidized_cut_copper"
    OXIDIZED_CUT_COPPER_SLAB = "minecraft:oxidized_cut_copper_slab"
    OXIDIZED_CUT_COPPER_STAIRS = "minecraft:oxidized_cut_copper_stairs"
    OXIDIZED_LIGHTNING_ROD = "minecraft:oxidized_lightning_rod"
    PACKED_ICE = "minecraft:packed_ice"
    PACKED_MUD = "minecraft:packed_mud"
    PAINTING = "minecraft:painting"
    PALE_HANGING_MOSS = "minecraft:pale_hanging_moss"
    PALE_MOSS_BLOCK = "minecraft:pale_moss_block"
    PALE_MOSS_CARPET = "minecraft:pale_moss_carpet"
    PALE_OAK_BOAT = "minecraft:pale_oak_boat"
    PALE_OAK_BUTTON = "minecraft:pale_oak_button"
    PALE_OAK_CHEST_BOAT = "minecraft:pale_oak_chest_boat"
    PALE_OAK_DOOR = "minecraft:pale_oak_door"
    PALE_OAK_FENCE_GATE = "minecraft:pale_oak_fence_gate"
    PALE_OAK_FENCE = "minecraft:pale_oak_fence"
    PALE_OAK_HANGING_SIGN = "minecraft:pale_oak_hanging_sign"
    PALE_OAK_LEAVES = "minecraft:pale_oak_leaves"
    PALE_OAK_LOG = "minecraft:pale_oak_log"
    PALE_OAK_PLANKS = "minecraft:pale_oak_planks"
    PALE_OAK_PRESSURE_PLATE = "minecraft:pale_oak_pressure_plate"
    PALE_OAK_SAPLING = "minecraft:pale_oak_sapling"
    PALE_OAK_SHELF = "minecraft:pale_oak_shelf"
    PALE_OAK_SIGN = "minecraft:pale_oak_sign"
    PALE_OAK_SLAB = "minecraft:pale_oak_slab"
    PALE_OAK_STAIRS = "minecraft:pale_oak_stairs"
    PALE_OAK_TRAPDOOR = "minecraft:pale_oak_trapdoor"
    PALE_OAK_WOOD = "minecraft:pale_oak_wood"
    PANDA_SPAWN_EGG = "minecraft:panda_spawn_egg"
    PAPER = "minecraft:paper"
    PARCHED_SPAWN_EGG = "minecraft:parched_spawn_egg"
    PARROT_SPAWN_EGG = "minecraft:parrot_spawn_egg"
    PEARLESCENT_FROGLIGHT = "minecraft:pearlescent_froglight"
    PEONY = "minecraft:peony"
    PETRIFIED_OAK_SLAB = "minecraft:petrified_oak_slab"
    PHANTOM_MEMBRANE = "minecraft:phantom_membrane"
    PHANTOM_SPAWN_EGG = "minecraft:phantom_spawn_egg"
    PIGLIN_BANNER_PATTERN = "minecraft:piglin_banner_pattern"
    PIGLIN_BRUTE_SPAWN_EGG = "minecraft:piglin_brute_spawn_egg"
    PIGLIN_HEAD = "minecraft:piglin_head"
    PIGLIN_SPAWN_EGG = "minecraft:piglin_spawn_egg"
    PIG_SPAWN_EGG = "minecraft:pig_spawn_egg"
    PILLAGER_SPAWN_EGG = "minecraft:pillager_spawn_egg"
    PINK_BANNER = "minecraft:pink_banner"
    PINK_BED = "minecraft:pink_bed"
    PINK_BUNDLE = "minecraft:pink_bundle"
    PINK_CANDLE = "minecraft:pink_candle"
    PINK_CARPET = "minecraft:pink_carpet"
    PINK_CONCRETE = "minecraft:pink_concrete"
    PINK_CONCRETE_POWDER = "minecraft:pink_concrete_powder"
    PINK_DYE = "minecraft:pink_dye"
    PINK_GLAZED_TERRACOTTA = "minecraft:pink_glazed_terracotta"
    PINK_HARNESS = "minecraft:pink_harness"
    PINK_PETALS = "minecraft:pink_petals"
    PINK_SHULKER_BOX = "minecraft:pink_shulker_box"
    PINK_STAINED_GLASS = "minecraft:pink_stained_glass"
    PINK_STAINED_GLASS_PANE = "minecraft:pink_stained_glass_pane"
    PINK_TERRACOTTA = "minecraft:pink_terracotta"
    PINK_TULIP = "minecraft:pink_tulip"
    PINK_WOOL = "minecraft:pink_wool"
    PISTON = "minecraft:piston"
    PITCHER_PLANT = "minecraft:pitcher_plant"
    PITCHER_POD = "minecraft:pitcher_pod"
    PLAYER_HEAD = "minecraft:player_head"
    PLENTY_POTTERY_SHERD = "minecraft:plenty_pottery_sherd"
    PODZOL = "minecraft:podzol"
    POINTED_DRIPSTONE = "minecraft:pointed_dripstone"
    POISONOUS_POTATO = "minecraft:poisonous_potato"
    POLAR_BEAR_SPAWN_EGG = "minecraft:polar_bear_spawn_egg"
    POLISHED_ANDESITE = "minecraft:polished_andesite"
    POLISHED_ANDESITE_SLAB = "minecraft:polished_andesite_slab"
    POLISHED_ANDESITE_STAIRS = "minecraft:polished_andesite_stairs"
    POLISHED_BASALT = "minecraft:polished_basalt"
    POLISHED_BLACKSTONE_BRICKS = "minecraft:polished_blackstone_bricks"
    POLISHED_BLACKSTONE_BRICK_SLAB = "minecraft:polished_blackstone_brick_slab"
    POLISHED_BLACKSTONE_BRICK_STAIRS = "minecraft:polished_blackstone_brick_stairs"
    POLISHED_BLACKSTONE_BRICK_WALL = "minecraft:polished_blackstone_brick_wall"
    POLISHED_BLACKSTONE_BUTTON = "minecraft:polished_blackstone_button"
    POLISHED_BLACKSTONE = "minecraft:polished_blackstone"
    POLISHED_BLACKSTONE_PRESSURE_PLATE = "minecraft:polished_blackstone_pressure_plate"
    POLISHED_BLACKSTONE_SLAB = "minecraft:polished_blackstone_slab"
    POLISHED_BLACKSTONE_STAIRS = "minecraft:polished_blackstone_stairs"
    POLISHED_BLACKSTONE_WALL = "minecraft:polished_blackstone_wall"
    POLISHED_DEEPSLATE = "minecraft:polished_deepslate"
    POLISHED_DEEPSLATE_SLAB = "minecraft:polished_deepslate_slab"
    POLISHED_DEEPSLATE_STAIRS = "minecraft:polished_deepslate_stairs"
    POLISHED_DEEPSLATE_WALL = "minecraft:polished_deepslate_wall"
    POLISHED_DIORITE = "minecraft:polished_diorite"
    POLISHED_DIORITE_SLAB = "minecraft:polished_diorite_slab"
    POLISHED_DIORITE_STAIRS = "minecraft:polished_diorite_stairs"
    POLISHED_GRANITE = "minecraft:polished_granite"
    POLISHED_GRANITE_SLAB = "minecraft:polished_granite_slab"
    POLISHED_GRANITE_STAIRS = "minecraft:polished_granite_stairs"
    POLISHED_TUFF = "minecraft:polished_tuff"
    POLISHED_TUFF_SLAB = "minecraft:polished_tuff_slab"
    POLISHED_TUFF_STAIRS = "minecraft:polished_tuff_stairs"
    POLISHED_TUFF_WALL = "minecraft:polished_tuff_wall"
    POPPED_CHORUS_FRUIT = "minecraft:popped_chorus_fruit"
    POPPY = "minecraft:poppy"
    PORKCHOP = "minecraft:porkchop"
    POTATO = "minecraft:potato"
    POTION = "minecraft:potion"
    POWDER_SNOW_BUCKET = "minecraft:powder_snow_bucket"
    POWERED_RAIL = "minecraft:powered_rail"
    PRISMARINE_BRICKS = "minecraft:prismarine_bricks"
    PRISMARINE_BRICK_SLAB = "minecraft:prismarine_brick_slab"
    PRISMARINE_BRICK_STAIRS = "minecraft:prismarine_brick_stairs"
    PRISMARINE_CRYSTALS = "minecraft:prismarine_crystals"
    PRISMARINE = "minecraft:prismarine"
    PRISMARINE_SHARD = "minecraft:prismarine_shard"
    PRISMARINE_SLAB = "minecraft:prismarine_slab"
    PRISMARINE_STAIRS = "minecraft:prismarine_stairs"
    PRISMARINE_WALL = "minecraft:prismarine_wall"
    PRIZE_POTTERY_SHERD = "minecraft:prize_pottery_sherd"
    PUFFERFISH_BUCKET = "minecraft:pufferfish_bucket"
    PUFFERFISH = "minecraft:pufferfish"
    PUFFERFISH_SPAWN_EGG = "minecraft:pufferfish_spawn_egg"
    PUMPKIN = "minecraft:pumpkin"
    PUMPKIN_PIE = "minecraft:pumpkin_pie"
    PUMPKIN_SEEDS = "minecraft:pumpkin_seeds"
    PURPLE_BANNER = "minecraft:purple_banner"
    PURPLE_BED = "minecraft:purple_bed"
    PURPLE_BUNDLE = "minecraft:purple_bundle"
    PURPLE_CANDLE = "minecraft:purple_candle"
    PURPLE_CARPET = "minecraft:purple_carpet"
    PURPLE_CONCRETE = "minecraft:purple_concrete"
    PURPLE_CONCRETE_POWDER = "minecraft:purple_concrete_powder"
    PURPLE_DYE = "minecraft:purple_dye"
    PURPLE_GLAZED_TERRACOTTA = "minecraft:purple_glazed_terracotta"
    PURPLE_HARNESS = "minecraft:purple_harness"
    PURPLE_SHULKER_BOX = "minecraft:purple_shulker_box"
    PURPLE_STAINED_GLASS = "minecraft:purple_stained_glass"
    PURPLE_STAINED_GLASS_PANE = "minecraft:purple_stained_glass_pane"
    PURPLE_TERRACOTTA = "minecraft:purple_terracotta"
    PURPLE_WOOL = "minecraft:purple_wool"
    PURPUR_BLOCK = "minecraft:purpur_block"
    PURPUR_PILLAR = "minecraft:purpur_pillar"
    PURPUR_SLAB = "minecraft:purpur_slab"
    PURPUR_STAIRS = "minecraft:purpur_stairs"
    QUARTZ_BLOCK = "minecraft:quartz_block"
    QUARTZ_BRICKS = "minecraft:quartz_bricks"
    QUARTZ = "minecraft:quartz"
    QUARTZ_PILLAR = "minecraft:quartz_pillar"
    QUARTZ_SLAB = "minecraft:quartz_slab"
    QUARTZ_STAIRS = "minecraft:quartz_stairs"
    RABBIT_FOOT = "minecraft:rabbit_foot"
    RABBIT_HIDE = "minecraft:rabbit_hide"
    RABBIT = "minecraft:rabbit"
    RABBIT_SPAWN_EGG = "minecraft:rabbit_spawn_egg"
    RABBIT_STEW = "minecraft:rabbit_stew"
    RAIL = "minecraft:rail"
    RAISER_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:raiser_armor_trim_smithing_template"
    RAVAGER_SPAWN_EGG = "minecraft:ravager_spawn_egg"
    RAW_COPPER_BLOCK = "minecraft:raw_copper_block"
    RAW_COPPER = "minecraft:raw_copper"
    RAW_GOLD_BLOCK = "minecraft:raw_gold_block"
    RAW_GOLD = "minecraft:raw_gold"
    RAW_IRON_BLOCK = "minecraft:raw_iron_block"
    RAW_IRON = "minecraft:raw_iron"
    RECOVERY_COMPASS = "minecraft:recovery_compass"
    RED_BANNER = "minecraft:red_banner"
    RED_BED = "minecraft:red_bed"
    RED_BUNDLE = "minecraft:red_bundle"
    RED_CANDLE = "minecraft:red_candle"
    RED_CARPET = "minecraft:red_carpet"
    RED_CONCRETE = "minecraft:red_concrete"
    RED_CONCRETE_POWDER = "minecraft:red_concrete_powder"
    RED_DYE = "minecraft:red_dye"
    RED_GLAZED_TERRACOTTA = "minecraft:red_glazed_terracotta"
    RED_HARNESS = "minecraft:red_harness"
    RED_MUSHROOM_BLOCK = "minecraft:red_mushroom_block"
    RED_MUSHROOM = "minecraft:red_mushroom"
    RED_NETHER_BRICKS = "minecraft:red_nether_bricks"
    RED_NETHER_BRICK_SLAB = "minecraft:red_nether_brick_slab"
    RED_NETHER_BRICK_STAIRS = "minecraft:red_nether_brick_stairs"
    RED_NETHER_BRICK_WALL = "minecraft:red_nether_brick_wall"
    RED_SAND = "minecraft:red_sand"
    RED_SANDSTONE = "minecraft:red_sandstone"
    RED_SANDSTONE_SLAB = "minecraft:red_sandstone_slab"
    RED_SANDSTONE_STAIRS = "minecraft:red_sandstone_stairs"
    RED_SANDSTONE_WALL = "minecraft:red_sandstone_wall"
    RED_SHULKER_BOX = "minecraft:red_shulker_box"
    RED_STAINED_GLASS = "minecraft:red_stained_glass"
    RED_STAINED_GLASS_PANE = "minecraft:red_stained_glass_pane"
    REDSTONE_BLOCK = "minecraft:redstone_block"
    REDSTONE = "minecraft:redstone"
    REDSTONE_LAMP = "minecraft:redstone_lamp"
    REDSTONE_ORE = "minecraft:redstone_ore"
    REDSTONE_TORCH = "minecraft:redstone_torch"
    RED_TERRACOTTA = "minecraft:red_terracotta"
    RED_TULIP = "minecraft:red_tulip"
    RED_WOOL = "minecraft:red_wool"
    REINFORCED_DEEPSLATE = "minecraft:reinforced_deepslate"
    REPEATER = "minecraft:repeater"
    REPEATING_COMMAND_BLOCK = "minecraft:repeating_command_block"
    RESIN_BLOCK = "minecraft:resin_block"
    RESIN_BRICK = "minecraft:resin_brick"
    RESIN_BRICKS = "minecraft:resin_bricks"
    RESIN_BRICK_SLAB = "minecraft:resin_brick_slab"
    RESIN_BRICK_STAIRS = "minecraft:resin_brick_stairs"
    RESIN_BRICK_WALL = "minecraft:resin_brick_wall"
    RESIN_CLUMP = "minecraft:resin_clump"
    RESPAWN_ANCHOR = "minecraft:respawn_anchor"
    RIB_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:rib_armor_trim_smithing_template"
    ROOTED_DIRT = "minecraft:rooted_dirt"
    ROSE_BUSH = "minecraft:rose_bush"
    ROTTEN_FLESH = "minecraft:rotten_flesh"
    SADDLE = "minecraft:saddle"
    SALMON_BUCKET = "minecraft:salmon_bucket"
    SALMON = "minecraft:salmon"
    SALMON_SPAWN_EGG = "minecraft:salmon_spawn_egg"
    SAND = "minecraft:sand"
    SANDSTONE = "minecraft:sandstone"
    SANDSTONE_SLAB = "minecraft:sandstone_slab"
    SANDSTONE_STAIRS = "minecraft:sandstone_stairs"
    SANDSTONE_WALL = "minecraft:sandstone_wall"
    SCAFFOLDING = "minecraft:scaffolding"
    SCRAPE_POTTERY_SHERD = "minecraft:scrape_pottery_sherd"
    SCULK_CATALYST = "minecraft:sculk_catalyst"
    SCULK = "minecraft:sculk"
    SCULK_SENSOR = "minecraft:sculk_sensor"
    SCULK_SHRIEKER = "minecraft:sculk_shrieker"
    SCULK_VEIN = "minecraft:sculk_vein"
    SEAGRASS = "minecraft:seagrass"
    SEA_LANTERN = "minecraft:sea_lantern"
    SEA_PICKLE = "minecraft:sea_pickle"
    SENTRY_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:sentry_armor_trim_smithing_template"
    SHAPER_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:shaper_armor_trim_smithing_template"
    SHEAF_POTTERY_SHERD = "minecraft:sheaf_pottery_sherd"
    SHEARS = "minecraft:shears"
    SHEEP_SPAWN_EGG = "minecraft:sheep_spawn_egg"
    SHELTER_POTTERY_SHERD = "minecraft:shelter_pottery_sherd"
    SHIELD = "minecraft:shield"
    SHORT_DRY_GRASS = "minecraft:short_dry_grass"
    SHORT_GRASS = "minecraft:short_grass"
    SHROOMLIGHT = "minecraft:shroomlight"
    SHULKER_BOX = "minecraft:shulker_box"
    SHULKER_SHELL = "minecraft:shulker_shell"
    SHULKER_SPAWN_EGG = "minecraft:shulker_spawn_egg"
    SILENCE_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:silence_armor_trim_smithing_template"
    SILVERFISH_SPAWN_EGG = "minecraft:silverfish_spawn_egg"
    SKELETON_HORSE_SPAWN_EGG = "minecraft:skeleton_horse_spawn_egg"
    SKELETON_SKULL = "minecraft:skeleton_skull"
    SKELETON_SPAWN_EGG = "minecraft:skeleton_spawn_egg"
    SKULL_BANNER_PATTERN = "minecraft:skull_banner_pattern"
    SKULL_POTTERY_SHERD = "minecraft:skull_pottery_sherd"
    SLIME_BALL = "minecraft:slime_ball"
    SLIME_BLOCK = "minecraft:slime_block"
    SLIME_SPAWN_EGG = "minecraft:slime_spawn_egg"
    SMALL_AMETHYST_BUD = "minecraft:small_amethyst_bud"
    SMALL_DRIPLEAF = "minecraft:small_dripleaf"
    SMITHING_TABLE = "minecraft:smithing_table"
    SMOKER = "minecraft:smoker"
    SMOOTH_BASALT = "minecraft:smooth_basalt"
    SMOOTH_QUARTZ = "minecraft:smooth_quartz"
    SMOOTH_QUARTZ_SLAB = "minecraft:smooth_quartz_slab"
    SMOOTH_QUARTZ_STAIRS = "minecraft:smooth_quartz_stairs"
    SMOOTH_RED_SANDSTONE = "minecraft:smooth_red_sandstone"
    SMOOTH_RED_SANDSTONE_SLAB = "minecraft:smooth_red_sandstone_slab"
    SMOOTH_RED_SANDSTONE_STAIRS = "minecraft:smooth_red_sandstone_stairs"
    SMOOTH_SANDSTONE = "minecraft:smooth_sandstone"
    SMOOTH_SANDSTONE_SLAB = "minecraft:smooth_sandstone_slab"
    SMOOTH_SANDSTONE_STAIRS = "minecraft:smooth_sandstone_stairs"
    SMOOTH_STONE = "minecraft:smooth_stone"
    SMOOTH_STONE_SLAB = "minecraft:smooth_stone_slab"
    SNIFFER_EGG = "minecraft:sniffer_egg"
    SNIFFER_SPAWN_EGG = "minecraft:sniffer_spawn_egg"
    SNORT_POTTERY_SHERD = "minecraft:snort_pottery_sherd"
    SNOUT_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:snout_armor_trim_smithing_template"
    SNOWBALL = "minecraft:snowball"
    SNOW_BLOCK = "minecraft:snow_block"
    SNOW_GOLEM_SPAWN_EGG = "minecraft:snow_golem_spawn_egg"
    SNOW = "minecraft:snow"
    SOUL_CAMPFIRE = "minecraft:soul_campfire"
    SOUL_LANTERN = "minecraft:soul_lantern"
    SOUL_SAND = "minecraft:soul_sand"
    SOUL_SOIL = "minecraft:soul_soil"
    SOUL_TORCH = "minecraft:soul_torch"
    SPAWNER = "minecraft:spawner"
    SPECTRAL_ARROW = "minecraft:spectral_arrow"
    SPIDER_EYE = "minecraft:spider_eye"
    SPIDER_SPAWN_EGG = "minecraft:spider_spawn_egg"
    SPIRE_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:spire_armor_trim_smithing_template"
    SPLASH_POTION = "minecraft:splash_potion"
    SPONGE = "minecraft:sponge"
    SPORE_BLOSSOM = "minecraft:spore_blossom"
    SPRUCE_BOAT = "minecraft:spruce_boat"
    SPRUCE_BUTTON = "minecraft:spruce_button"
    SPRUCE_CHEST_BOAT = "minecraft:spruce_chest_boat"
    SPRUCE_DOOR = "minecraft:spruce_door"
    SPRUCE_FENCE_GATE = "minecraft:spruce_fence_gate"
    SPRUCE_FENCE = "minecraft:spruce_fence"
    SPRUCE_HANGING_SIGN = "minecraft:spruce_hanging_sign"
    SPRUCE_LEAVES = "minecraft:spruce_leaves"
    SPRUCE_LOG = "minecraft:spruce_log"
    SPRUCE_PLANKS = "minecraft:spruce_planks"
    SPRUCE_PRESSURE_PLATE = "minecraft:spruce_pressure_plate"
    SPRUCE_SAPLING = "minecraft:spruce_sapling"
    SPRUCE_SHELF = "minecraft:spruce_shelf"
    SPRUCE_SIGN = "minecraft:spruce_sign"
    SPRUCE_SLAB = "minecraft:spruce_slab"
    SPRUCE_STAIRS = "minecraft:spruce_stairs"
    SPRUCE_TRAPDOOR = "minecraft:spruce_trapdoor"
    SPRUCE_WOOD = "minecraft:spruce_wood"
    SPYGLASS = "minecraft:spyglass"
    SQUID_SPAWN_EGG = "minecraft:squid_spawn_egg"
    STICK = "minecraft:stick"
    STICKY_PISTON = "minecraft:sticky_piston"
    STONE_AXE = "minecraft:stone_axe"
    STONE_BRICKS = "minecraft:stone_bricks"
    STONE_BRICK_SLAB = "minecraft:stone_brick_slab"
    STONE_BRICK_STAIRS = "minecraft:stone_brick_stairs"
    STONE_BRICK_WALL = "minecraft:stone_brick_wall"
    STONE_BUTTON = "minecraft:stone_button"
    STONECUTTER = "minecraft:stonecutter"
    STONE_HOE = "minecraft:stone_hoe"
    STONE = "minecraft:stone"
    STONE_PICKAXE = "minecraft:stone_pickaxe"
    STONE_PRESSURE_PLATE = "minecraft:stone_pressure_plate"
    STONE_SHOVEL = "minecraft:stone_shovel"
    STONE_SLAB = "minecraft:stone_slab"
    STONE_SPEAR = "minecraft:stone_spear"
    STONE_STAIRS = "minecraft:stone_stairs"
    STONE_SWORD = "minecraft:stone_sword"
    STRAY_SPAWN_EGG = "minecraft:stray_spawn_egg"
    STRIDER_SPAWN_EGG = "minecraft:strider_spawn_egg"
    STRING = "minecraft:string"
    STRIPPED_ACACIA_LOG = "minecraft:stripped_acacia_log"
    STRIPPED_ACACIA_WOOD = "minecraft:stripped_acacia_wood"
    STRIPPED_BAMBOO_BLOCK = "minecraft:stripped_bamboo_block"
    STRIPPED_BIRCH_LOG = "minecraft:stripped_birch_log"
    STRIPPED_BIRCH_WOOD = "minecraft:stripped_birch_wood"
    STRIPPED_CHERRY_LOG = "minecraft:stripped_cherry_log"
    STRIPPED_CHERRY_WOOD = "minecraft:stripped_cherry_wood"
    STRIPPED_CRIMSON_HYPHAE = "minecraft:stripped_crimson_hyphae"
    STRIPPED_CRIMSON_STEM = "minecraft:stripped_crimson_stem"
    STRIPPED_DARK_OAK_LOG = "minecraft:stripped_dark_oak_log"
    STRIPPED_DARK_OAK_WOOD = "minecraft:stripped_dark_oak_wood"
    STRIPPED_JUNGLE_LOG = "minecraft:stripped_jungle_log"
    STRIPPED_JUNGLE_WOOD = "minecraft:stripped_jungle_wood"
    STRIPPED_MANGROVE_LOG = "minecraft:stripped_mangrove_log"
    STRIPPED_MANGROVE_WOOD = "minecraft:stripped_mangrove_wood"
    STRIPPED_OAK_LOG = "minecraft:stripped_oak_log"
    STRIPPED_OAK_WOOD = "minecraft:stripped_oak_wood"
    STRIPPED_PALE_OAK_LOG = "minecraft:stripped_pale_oak_log"
    STRIPPED_PALE_OAK_WOOD = "minecraft:stripped_pale_oak_wood"
    STRIPPED_SPRUCE_LOG = "minecraft:stripped_spruce_log"
    STRIPPED_SPRUCE_WOOD = "minecraft:stripped_spruce_wood"
    STRIPPED_WARPED_HYPHAE = "minecraft:stripped_warped_hyphae"
    STRIPPED_WARPED_STEM = "minecraft:stripped_warped_stem"
    STRUCTURE_BLOCK = "minecraft:structure_block"
    STRUCTURE_VOID = "minecraft:structure_void"
    SUGAR_CANE = "minecraft:sugar_cane"
    SUGAR = "minecraft:sugar"
    SUNFLOWER = "minecraft:sunflower"
    SUSPICIOUS_GRAVEL = "minecraft:suspicious_gravel"
    SUSPICIOUS_SAND = "minecraft:suspicious_sand"
    SUSPICIOUS_STEW = "minecraft:suspicious_stew"
    SWEET_BERRIES = "minecraft:sweet_berries"
    TADPOLE_BUCKET = "minecraft:tadpole_bucket"
    TADPOLE_SPAWN_EGG = "minecraft:tadpole_spawn_egg"
    TALL_DRY_GRASS = "minecraft:tall_dry_grass"
    TALL_GRASS = "minecraft:tall_grass"
    TARGET = "minecraft:target"
    TERRACOTTA = "minecraft:terracotta"
    TEST_BLOCK = "minecraft:test_block"
    TEST_INSTANCE_BLOCK = "minecraft:test_instance_block"
    TEST = "minecraft:test"
    TIDE_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:tide_armor_trim_smithing_template"
    TINTED_GLASS = "minecraft:tinted_glass"
    TIPPED_ARROW = "minecraft:tipped_arrow"
    TNT = "minecraft:tnt"
    TNT_MINECART = "minecraft:tnt_minecart"
    TORCHFLOWER = "minecraft:torchflower"
    TORCHFLOWER_SEEDS = "minecraft:torchflower_seeds"
    TORCH = "minecraft:torch"
    TOTEM_OF_UNDYING = "minecraft:totem_of_undying"
    TRADER_LLAMA_SPAWN_EGG = "minecraft:trader_llama_spawn_egg"
    TRAPPED_CHEST = "minecraft:trapped_chest"
    TRIAL_KEY = "minecraft:trial_key"
    TRIAL_SPAWNER = "minecraft:trial_spawner"
    TRIDENT = "minecraft:trident"
    TRIPWIRE_HOOK = "minecraft:tripwire_hook"
    TROPICAL_FISH_BUCKET = "minecraft:tropical_fish_bucket"
    TROPICAL_FISH = "minecraft:tropical_fish"
    TROPICAL_FISH_SPAWN_EGG = "minecraft:tropical_fish_spawn_egg"
    TUBE_CORAL_BLOCK = "minecraft:tube_coral_block"
    TUBE_CORAL_FAN = "minecraft:tube_coral_fan"
    TUBE_CORAL = "minecraft:tube_coral"
    TUFF_BRICKS = "minecraft:tuff_bricks"
    TUFF_BRICK_SLAB = "minecraft:tuff_brick_slab"
    TUFF_BRICK_STAIRS = "minecraft:tuff_brick_stairs"
    TUFF_BRICK_WALL = "minecraft:tuff_brick_wall"
    TUFF = "minecraft:tuff"
    TUFF_SLAB = "minecraft:tuff_slab"
    TUFF_STAIRS = "minecraft:tuff_stairs"
    TUFF_WALL = "minecraft:tuff_wall"
    TURTLE_EGG = "minecraft:turtle_egg"
    TURTLE_HELMET = "minecraft:turtle_helmet"
    TURTLE_SCUTE = "minecraft:turtle_scute"
    TURTLE_SPAWN_EGG = "minecraft:turtle_spawn_egg"
    TWISTING_VINES = "minecraft:twisting_vines"
    VAULT = "minecraft:vault"
    VERDANT_FROGLIGHT = "minecraft:verdant_froglight"
    VEX_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:vex_armor_trim_smithing_template"
    VEX_SPAWN_EGG = "minecraft:vex_spawn_egg"
    VILLAGER_SPAWN_EGG = "minecraft:villager_spawn_egg"
    VINDICATOR_SPAWN_EGG = "minecraft:vindicator_spawn_egg"
    VINE = "minecraft:vine"
    WANDERING_TRADER_SPAWN_EGG = "minecraft:wandering_trader_spawn_egg"
    WARD_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:ward_armor_trim_smithing_template"
    WARDEN_SPAWN_EGG = "minecraft:warden_spawn_egg"
    WARPED_BUTTON = "minecraft:warped_button"
    WARPED_DOOR = "minecraft:warped_door"
    WARPED_FENCE_GATE = "minecraft:warped_fence_gate"
    WARPED_FENCE = "minecraft:warped_fence"
    WARPED_FUNGUS = "minecraft:warped_fungus"
    WARPED_FUNGUS_ON_A_STICK = "minecraft:warped_fungus_on_a_stick"
    WARPED_HANGING_SIGN = "minecraft:warped_hanging_sign"
    WARPED_HYPHAE = "minecraft:warped_hyphae"
    WARPED_NYLIUM = "minecraft:warped_nylium"
    WARPED_PLANKS = "minecraft:warped_planks"
    WARPED_PRESSURE_PLATE = "minecraft:warped_pressure_plate"
    WARPED_ROOTS = "minecraft:warped_roots"
    WARPED_SHELF = "minecraft:warped_shelf"
    WARPED_SIGN = "minecraft:warped_sign"
    WARPED_SLAB = "minecraft:warped_slab"
    WARPED_STAIRS = "minecraft:warped_stairs"
    WARPED_STEM = "minecraft:warped_stem"
    WARPED_TRAPDOOR = "minecraft:warped_trapdoor"
    WARPED_WART_BLOCK = "minecraft:warped_wart_block"
    WATER_BUCKET = "minecraft:water_bucket"
    WAXED_CHISELED_COPPER = "minecraft:waxed_chiseled_copper"
    WAXED_COPPER_BARS = "minecraft:waxed_copper_bars"
    WAXED_COPPER_BLOCK = "minecraft:waxed_copper_block"
    WAXED_COPPER_BULB = "minecraft:waxed_copper_bulb"
    WAXED_COPPER_CHAIN = "minecraft:waxed_copper_chain"
    WAXED_COPPER_CHEST = "minecraft:waxed_copper_chest"
    WAXED_COPPER_DOOR = "minecraft:waxed_copper_door"
    WAXED_COPPER_GOLEM_STATUE = "minecraft:waxed_copper_golem_statue"
    WAXED_COPPER_GRATE = "minecraft:waxed_copper_grate"
    WAXED_COPPER_LANTERN = "minecraft:waxed_copper_lantern"
    WAXED_COPPER_TRAPDOOR = "minecraft:waxed_copper_trapdoor"
    WAXED_CUT_COPPER = "minecraft:waxed_cut_copper"
    WAXED_CUT_COPPER_SLAB = "minecraft:waxed_cut_copper_slab"
    WAXED_CUT_COPPER_STAIRS = "minecraft:waxed_cut_copper_stairs"
    WAXED_EXPOSED_CHISELED_COPPER = "minecraft:waxed_exposed_chiseled_copper"
    WAXED_EXPOSED_COPPER_BARS = "minecraft:waxed_exposed_copper_bars"
    WAXED_EXPOSED_COPPER_BULB = "minecraft:waxed_exposed_copper_bulb"
    WAXED_EXPOSED_COPPER_CHAIN = "minecraft:waxed_exposed_copper_chain"
    WAXED_EXPOSED_COPPER_CHEST = "minecraft:waxed_exposed_copper_chest"
    WAXED_EXPOSED_COPPER_DOOR = "minecraft:waxed_exposed_copper_door"
    WAXED_EXPOSED_COPPER_GOLEM_STATUE = "minecraft:waxed_exposed_copper_golem_statue"
    WAXED_EXPOSED_COPPER_GRATE = "minecraft:waxed_exposed_copper_grate"
    WAXED_EXPOSED_COPPER = "minecraft:waxed_exposed_copper"
    WAXED_EXPOSED_COPPER_LANTERN = "minecraft:waxed_exposed_copper_lantern"
    WAXED_EXPOSED_COPPER_TRAPDOOR = "minecraft:waxed_exposed_copper_trapdoor"
    WAXED_EXPOSED_CUT_COPPER = "minecraft:waxed_exposed_cut_copper"
    WAXED_EXPOSED_CUT_COPPER_SLAB = "minecraft:waxed_exposed_cut_copper_slab"
    WAXED_EXPOSED_CUT_COPPER_STAIRS = "minecraft:waxed_exposed_cut_copper_stairs"
    WAXED_EXPOSED_LIGHTNING_ROD = "minecraft:waxed_exposed_lightning_rod"
    WAXED_LIGHTNING_ROD = "minecraft:waxed_lightning_rod"
    WAXED_OXIDIZED_CHISELED_COPPER = "minecraft:waxed_oxidized_chiseled_copper"
    WAXED_OXIDIZED_COPPER_BARS = "minecraft:waxed_oxidized_copper_bars"
    WAXED_OXIDIZED_COPPER_BULB = "minecraft:waxed_oxidized_copper_bulb"
    WAXED_OXIDIZED_COPPER_CHAIN = "minecraft:waxed_oxidized_copper_chain"
    WAXED_OXIDIZED_COPPER_CHEST = "minecraft:waxed_oxidized_copper_chest"
    WAXED_OXIDIZED_COPPER_DOOR = "minecraft:waxed_oxidized_copper_door"
    WAXED_OXIDIZED_COPPER_GOLEM_STATUE = "minecraft:waxed_oxidized_copper_golem_statue"
    WAXED_OXIDIZED_COPPER_GRATE = "minecraft:waxed_oxidized_copper_grate"
    WAXED_OXIDIZED_COPPER = "minecraft:waxed_oxidized_copper"
    WAXED_OXIDIZED_COPPER_LANTERN = "minecraft:waxed_oxidized_copper_lantern"
    WAXED_OXIDIZED_COPPER_TRAPDOOR = "minecraft:waxed_oxidized_copper_trapdoor"
    WAXED_OXIDIZED_CUT_COPPER = "minecraft:waxed_oxidized_cut_copper"
    WAXED_OXIDIZED_CUT_COPPER_SLAB = "minecraft:waxed_oxidized_cut_copper_slab"
    WAXED_OXIDIZED_CUT_COPPER_STAIRS = "minecraft:waxed_oxidized_cut_copper_stairs"
    WAXED_OXIDIZED_LIGHTNING_ROD = "minecraft:waxed_oxidized_lightning_rod"
    WAXED_WEATHERED_CHISELED_COPPER = "minecraft:waxed_weathered_chiseled_copper"
    WAXED_WEATHERED_COPPER_BARS = "minecraft:waxed_weathered_copper_bars"
    WAXED_WEATHERED_COPPER_BULB = "minecraft:waxed_weathered_copper_bulb"
    WAXED_WEATHERED_COPPER_CHAIN = "minecraft:waxed_weathered_copper_chain"
    WAXED_WEATHERED_COPPER_CHEST = "minecraft:waxed_weathered_copper_chest"
    WAXED_WEATHERED_COPPER_DOOR = "minecraft:waxed_weathered_copper_door"
    WAXED_WEATHERED_COPPER_GOLEM_STATUE = "minecraft:waxed_weathered_copper_golem_statue"
    WAXED_WEATHERED_COPPER_GRATE = "minecraft:waxed_weathered_copper_grate"
    WAXED_WEATHERED_COPPER = "minecraft:waxed_weathered_copper"
    WAXED_WEATHERED_COPPER_LANTERN = "minecraft:waxed_weathered_copper_lantern"
    WAXED_WEATHERED_COPPER_TRAPDOOR = "minecraft:waxed_weathered_copper_trapdoor"
    WAXED_WEATHERED_CUT_COPPER = "minecraft:waxed_weathered_cut_copper"
    WAXED_WEATHERED_CUT_COPPER_SLAB = "minecraft:waxed_weathered_cut_copper_slab"
    WAXED_WEATHERED_CUT_COPPER_STAIRS = "minecraft:waxed_weathered_cut_copper_stairs"
    WAXED_WEATHERED_LIGHTNING_ROD = "minecraft:waxed_weathered_lightning_rod"
    WAYFINDER_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:wayfinder_armor_trim_smithing_template"
    WEATHERED_CHISELED_COPPER = "minecraft:weathered_chiseled_copper"
    WEATHERED_COPPER_BARS = "minecraft:weathered_copper_bars"
    WEATHERED_COPPER_BULB = "minecraft:weathered_copper_bulb"
    WEATHERED_COPPER_CHAIN = "minecraft:weathered_copper_chain"
    WEATHERED_COPPER_CHEST = "minecraft:weathered_copper_chest"
    WEATHERED_COPPER_DOOR = "minecraft:weathered_copper_door"
    WEATHERED_COPPER_GOLEM_STATUE = "minecraft:weathered_copper_golem_statue"
    WEATHERED_COPPER_GRATE = "minecraft:weathered_copper_grate"
    WEATHERED_COPPER = "minecraft:weathered_copper"
    WEATHERED_COPPER_LANTERN = "minecraft:weathered_copper_lantern"
    WEATHERED_COPPER_TRAPDOOR = "minecraft:weathered_copper_trapdoor"
    WEATHERED_CUT_COPPER = "minecraft:weathered_cut_copper"
    WEATHERED_CUT_COPPER_SLAB = "minecraft:weathered_cut_copper_slab"
    WEATHERED_CUT_COPPER_STAIRS = "minecraft:weathered_cut_copper_stairs"
    WEATHERED_LIGHTNING_ROD = "minecraft:weathered_lightning_rod"
    WEEPING_VINES = "minecraft:weeping_vines"
    WET_SPONGE = "minecraft:wet_sponge"
    WHEAT = "minecraft:wheat"
    WHEAT_SEEDS = "minecraft:wheat_seeds"
    WHITE_BANNER = "minecraft:white_banner"
    WHITE_BED = "minecraft:white_bed"
    WHITE_BUNDLE = "minecraft:white_bundle"
    WHITE_CANDLE = "minecraft:white_candle"
    WHITE_CARPET = "minecraft:white_carpet"
    WHITE_CONCRETE = "minecraft:white_concrete"
    WHITE_CONCRETE_POWDER = "minecraft:white_concrete_powder"
    WHITE_DYE = "minecraft:white_dye"
    WHITE_GLAZED_TERRACOTTA = "minecraft:white_glazed_terracotta"
    WHITE_HARNESS = "minecraft:white_harness"
    WHITE_SHULKER_BOX = "minecraft:white_shulker_box"
    WHITE_STAINED_GLASS = "minecraft:white_stained_glass"
    WHITE_STAINED_GLASS_PANE = "minecraft:white_stained_glass_pane"
    WHITE_TERRACOTTA = "minecraft:white_terracotta"
    WHITE_TULIP = "minecraft:white_tulip"
    WHITE_WOOL = "minecraft:white_wool"
    WILD_ARMOR_TRIM_SMITHING_TEMPLATE = "minecraft:wild_armor_trim_smithing_template"
    WILDFLOWERS = "minecraft:wildflowers"
    WIND_CHARGE = "minecraft:wind_charge"
    WITCH_SPAWN_EGG = "minecraft:witch_spawn_egg"
    WITHER_ROSE = "minecraft:wither_rose"
    WITHER_SKELETON_SKULL = "minecraft:wither_skeleton_skull"
    WITHER_SKELETON_SPAWN_EGG = "minecraft:wither_skeleton_spawn_egg"
    WITHER_SPAWN_EGG = "minecraft:wither_spawn_egg"
    WOLF_ARMOR = "minecraft:wolf_armor"
    WOLF_SPAWN_EGG = "minecraft:wolf_spawn_egg"
    WOODEN_AXE = "minecraft:wooden_axe"
    WOODEN_HOE = "minecraft:wooden_hoe"
    WOODEN_PICKAXE = "minecraft:wooden_pickaxe"
    WOODEN_SHOVEL = "minecraft:wooden_shovel"
    WOODEN_SPEAR = "minecraft:wooden_spear"
    WOODEN_SWORD = "minecraft:wooden_sword"
    WRITABLE_BOOK = "minecraft:writable_book"
    WRITTEN_BOOK = "minecraft:written_book"
    YELLOW_BANNER = "minecraft:yellow_banner"
    YELLOW_BED = "minecraft:yellow_bed"
    YELLOW_BUNDLE = "minecraft:yellow_bundle"
    YELLOW_CANDLE = "minecraft:yellow_candle"
    YELLOW_CARPET = "minecraft:yellow_carpet"
    YELLOW_CONCRETE = "minecraft:yellow_concrete"
    YELLOW_CONCRETE_POWDER = "minecraft:yellow_concrete_powder"
    YELLOW_DYE = "minecraft:yellow_dye"
    YELLOW_GLAZED_TERRACOTTA = "minecraft:yellow_glazed_terracotta"
    YELLOW_HARNESS = "minecraft:yellow_harness"
    YELLOW_SHULKER_BOX = "minecraft:yellow_shulker_box"
    YELLOW_STAINED_GLASS = "minecraft:yellow_stained_glass"
    YELLOW_STAINED_GLASS_PANE = "minecraft:yellow_stained_glass_pane"
    YELLOW_TERRACOTTA = "minecraft:yellow_terracotta"
    YELLOW_WOOL = "minecraft:yellow_wool"
    ZOGLIN_SPAWN_EGG = "minecraft:zoglin_spawn_egg"
    ZOMBIE_HEAD = "minecraft:zombie_head"
    ZOMBIE_HORSE_SPAWN_EGG = "minecraft:zombie_horse_spawn_egg"
    ZOMBIE_NAUTILUS_SPAWN_EGG = "minecraft:zombie_nautilus_spawn_egg"
    ZOMBIE_SPAWN_EGG = "minecraft:zombie_spawn_egg"
    ZOMBIE_VILLAGER_SPAWN_EGG = "minecraft:zombie_villager_spawn_egg"
    ZOMBIFIED_PIGLIN_SPAWN_EGG = "minecraft:zombified_piglin_spawn_egg"



# This file contains the result classes used to express the result of a recipe.
# Some recipes support an amount of items, but some do not.
# We handle this using two different classes:
# 1) SimpleResult  -> No amount of item can be specified.
# 2) CountedResult -> Amount of item can be specified.

from dataclasses import dataclass

from MCpypack.item import Item

@dataclass
class SimpleResult:
    """
    Result of a recipe that does not contain an item count.
    """

    item_id: Item

    def to_dict(self) -> dict[str, str]:
        """
        Return the result as a dict.
        """

        return {
            "id": self.item_id.value,
        }

# CountedResult is not a dataclass.
# This is due to the need of validation for the count property.
# With a dataclass, the user would have to type:
#   CountedResult(item_id=item, _count=count)
# This is inconvenient and we want the user to be able to type:
#   CountedResult(item_id=item, count=count)
class CountedResult:
    """
    Result of a recipe that can contain an item count.
    """

    def __init__(self,
                 item_id: Item,
                 count: int = 1,
                 ) -> None:
        """
        Init a new recipe with a count.

        Parameters
        ----------
        item_id:
            Id of the item.
        count:
            Amount of the item.
        """

        self.item_id: Item = item_id
        self.count = count

    @property
    def count(self) -> int:
        return self._count

    @count.setter
    def count(self, value: int) -> None:
        if not 1 <= value <= 64:
            raise ValueError(f"count must be between 1 and 64, got {value}")
        self._count = value

    def to_dict(self) -> dict[str, str | int]:
        """
        Return the result as a dict.
        """

        result: dict[str, str | int] = {}

        result["id"] = self.item_id.value

        if self.count != 1:
            result["count"] = self.count

        return result



# Core stuff
from .core.datapack import Datapack
from .core.namespace import Namespace

__all__ = ["Datapack", "Namespace"]

# Recipes
from .recipe import CraftingShaped, CraftingShapeless, CampfireCooking, Smoking, Blasting, Smelting, Stonecutting

__all__ += [
    "CraftingShaped",
    "CraftingShapeless",
    "CampfireCooking",
    "Smelting",
    "Smoking",
    "Blasting",
    "Stonecutting",
]

# Times
from .recipe import Milliseconds, Seconds, Minutes, Hours, Time
__all__ += [
    "Milliseconds",
    "Seconds",
    "Minutes",
    "Hours",
    "Time"
]

# Result
from .recipe import SimpleResult, CountedResult
__all__ += [
    "SimpleResult",
    "CountedResult"
]

# Items
from .item import Item
__all__ += [
    "Item"
]



# __init__.py file of 'recipe' directory
# Export CraftingShaped, CraftingShapeless, CampfireCooking

from .crafting_shaped import CraftingShaped
from .crafting_shapeless import CraftingShapeless
from .campfire_cooking import CampfireCooking
from .furnace import Smelting, Blasting, Smoking
from .stonecutting import Stonecutting

__all__: list[str] = [
    "CraftingShaped",
    "CraftingShapeless",
    "CampfireCooking",
    "Smelting",
    "Blasting",
    "Smoking",
    "Stonecutting",
]

from .utils import Milliseconds, Seconds, Minutes, Hours, Time
__all__ += [
    "Milliseconds",
    "Seconds",
    "Minutes",
    "Hours",
    "Time"
]

from .utils import SimpleResult, CountedResult
__all__ += [
    "SimpleResult",
    "CountedResult"
]



# This file contains the Time class
# It also contains other time related classes

from dataclasses import dataclass
from typing import TypeAlias

# Real life time classes
@dataclass
class Milliseconds:
    value: int

@dataclass
class Seconds:
    value: int

@dataclass
class Minutes:
    value: int

@dataclass
class Hours:
    value: int

RealLifeTime: TypeAlias = Milliseconds | Seconds | Minutes | Hours

# Minecraft time class
@dataclass
class Ticks:
    value: int

MinecraftTime: TypeAlias = Ticks


class Time:
    """
    Represents time.
    Can be initialized with multiple real-life time units.
    """

    def __init__(self, *rlTimes: RealLifeTime) -> None:
        """
        Initialize time with real-life time values.

        Parameters
        ----------
        *rlTimes: RealLifeTime
            Real-life time units (Milliseconds, Seconds, Minutes, Hours)
        """

        self.milliseconds: Milliseconds = Milliseconds(0)
        self.seconds: Seconds = Seconds(0)
        self.minutes: Minutes = Minutes(0)
        self.hours: Hours = Hours(0)

        for t in rlTimes:
            match t:
                case Milliseconds():
                    self.milliseconds.value += t.value
                case Seconds():
                    self.seconds.value += t.value
                case Minutes():
                    self.minutes.value += t.value
                case Hours():
                    self.hours.value += t.value
                case _:
                    raise ValueError(f"Unsupported time unit: {t}")

    @property
    def ticks(self) -> Ticks:
        """
        Return time as Minecraft ticks.
        """

        total_ticks: Ticks =  Ticks(
            int(self.milliseconds.value /1000 *20)
            + (self.seconds.value *20)
            + (self.minutes.value *60 *20)
            + (self.hours.value *3600 *20)
        )

        if total_ticks.value < 1:
            raise ValueError("Total ticks must be at least 1.")

        return total_ticks



from pathlib import Path
from enum import Enum


def generate_enum(input_file: str, output_file: str) -> None:
    input_path: Path = Path(input_file)
    output_path: Path = Path(output_file)

    lines: list[str] = input_path.read_text(encoding="utf-8").splitlines()
    items: list[str] = [line.strip() for line in lines if line.strip()]

    with output_path.open("w", encoding="utf-8") as f:
        f.write("from enum import Enum\n\n\n")
        f.write("class Item(Enum):\n")

        for item in items:
            enum_name: str = item.upper()
            enum_value: str = f"minecraft:{item.lower()}"
            f.write(f"    {enum_name} = \"{enum_value}\"\n")


if __name__ == "__main__":
    generate_enum("list/1.21.11.txt", "final/f_1_21_11.py")



# This file contains the Recipe class used to create new recipes

from abc import ABC, abstractmethod
import json
from pathlib import Path
from typing import Any

class Recipe(ABC):
    """
    Recipe class for adding recipes.
    """

    @property
    @abstractmethod
    def TYPE(cls) -> str:
        """
        Return the type of the furnace recipe.
        """
        pass

    def __init__(self, name: str) -> None:
        """
        Set up 'self.config'.

        Parameters
        ----------
        name:
            Name of recipe.
        """

        self.name: str = name

        self.config: dict[str, Any] = {}

        self.config["type"] = self.TYPE

    def export(self, namespace_dir: Path):
        """
        Create recipe file inside namespace.

        Parameters
        ----------
        namespace_dir:
            Directory of the namespace.
        """

        # Recipe directory inside namespace
        recipe_dir: Path = namespace_dir / "recipe"
        recipe_dir.mkdir(parents=True, exist_ok=True)

        file_path: Path = recipe_dir / f"{self.name}.json"
        with open(file_path, "w", encoding="utf-8") as file:
            json.dump(self.config, file, indent=4)



# __init__.py file of 'recipe/utils/'
# Export Category and CategoryLike
# Export Group
# Export SimpleResult, CountedResult
# Export Experience
# Export Time, Milliseconds, Seconds, Minutes, and Hours

__all__: list[str] = []

from .category import Category, CategoryLike
__all__ += ["Category", "CategoryLike"]

from .group import Group
__all__ += ["Group"]

from .result import SimpleResult, CountedResult
__all__ += ["SimpleResult", "CountedResult"]

from .experience import Experience
__all__ += ["Experience"]

from .times import Time, Milliseconds, Seconds, Minutes, Hours
__all__ += ["Time", "Milliseconds", "Seconds", "Minutes", "Hours"]



# This file contains recipes for furnaces in Minecraft.
# It support all 3 types of furnaces which are:
# 1) Standard furnace -> Smelting recipes
# 2) Blast furnace    -> Blasting recipes
# 3) Smoker           -> Smoking recipes
# The 3 classes for each type inherit from the Furnace class.
# All of them have to implement the TYPE property.

from MCpypack.item import Item
from .recipe import Recipe
from .utils import Time, SimpleResult, Group, CategoryLike, Category, Experience

from abc import ABC, abstractmethod

class Furnace(Recipe, ABC):
    """
    Base class for furnace recipes.
    """

    @property
    @abstractmethod
    def TYPE(cls) -> str:
        """
        Return the type of the furnace recipe.
        """
        pass

    def __init__(self,
                name: str,
                ingredient: Item | list[Item],
                result: SimpleResult,
                cookingtime: Time | None = None,
                group: Group | None = None,
                category: CategoryLike = Category.MISC,
                experience: Experience | None = None,

                ) -> None:
        """
        Init new furnace recipe.

        Parameters
        ----------
        name:
            Name of the smelting recipe.
        ingredient:
            Ingredient of the recipe.
        result:
            Result of the smelting.
        cookingtime:
            Optional.
            Cookingtime in real-life time values.
        category:
            Recipe book category of the recipe.
        group:
            String identifier for grouping recipes.
        experience:
            Optional. The output experience of the recipe.
        """

        super().__init__(name)

        # Convert category to Category enum if it is a string
        # Ensure valid value if string
        category_final: str = str(Category.from_str(category))

        self.config["category"] = category_final

        if isinstance(ingredient, list):
            self.config["ingredient"] = [i.value for i in ingredient]
        else:
            self.config["ingredient"] = ingredient.value

        self.config["result"] = result.to_dict()

        if group:
            self.config["group"] = group

        if experience:
            self.config["experience"] = experience

        if cookingtime:
            self.config["cookingtime"] = cookingtime.ticks.value

class Smelting(Furnace):
    """
    Smelting recipe.
    """

    @property
    def TYPE(cls) -> str:
        return "minecraft:smelting"

class Blasting(Furnace):
    """
    Blasting recipe.
    """

    @property
    def TYPE(cls) -> str:
        return "minecraft:blasting"

class Smoking(Furnace):
    """
    Smoking recipe.
    """

    @property
    def TYPE(cls) -> str:
        return "minecraft:smoking"
